@@ -42,8 +42,10 @@
 #include <syslog.h>
 #include <glib.h>
 #include <clplumbing/cl_log.h>
+#include <clplumbing/cl_malloc.h>
 #include <hb_api.h>
 
+
 /* exit code */
 const int OK = 0,
 	  NORMAL_FAIL = 1,		/* such as the local node is down */
@@ -165,86 +167,58 @@ static int test(ll_cluster_t *hb, int argc, char ** argv, const char * optstr);
 static int general_simple_opt_deal(int argc, char ** argv, const char * optstr);
 
 typedef struct {
-	const char * name;
-	int (*func)(ll_cluster_t *hb, int, char **, const char *);
-	const char * optstr;
+	const char *	name;
+	int 		(*func)(ll_cluster_t *hb, int, char **, const char *);
+	const char *	optstr;
+	gboolean	needsignon;
 } cmd_t;
 
 static const size_t CMDS_MAX_LENGTH = 16;
 static gboolean FOR_HUMAN_READ = FALSE;
 static const cmd_t cmds[] = {
-	{ "hbstatus",      hbstatus, 	  "m" },
-	{ "listnodes",     listnodes, 	  "mpn" },
-	{ "nodestatus",    nodestatus, 	  "m" },
-	{ "nodetype",      nodetype, 	  "m" },
-	{ "listhblinks",   listhblinks,   "m" },
-	{ "hblinkstatus",  hblinkstatus,  "m" },
-	{ "clientstatus",  clientstatus,  "m" },
-	{ "rscstatus",     rscstatus, 	  "m"}, 
-	{ "hbparameter",   hbparameter,	  "mp:"},
-	{ "test",	   test,	  NULL},
+	{ "hbstatus",      hbstatus, 	  "m" ,		FALSE},
+	{ "listnodes",     listnodes, 	  "mpn",	TRUE},
+	{ "nodestatus",    nodestatus, 	  "m",		TRUE},
+	{ "nodetype",      nodetype, 	  "m",		TRUE },
+	{ "listhblinks",   listhblinks,   "m",		TRUE },
+	{ "hblinkstatus",  hblinkstatus,  "m",		TRUE },
+	{ "clientstatus",  clientstatus,  "m",		TRUE },
+	{ "rscstatus",     rscstatus, 	  "m",		TRUE}, 
+	{ "hbparameter",   hbparameter,	  "mp:,		TRUE"},
+	{ "test",	   test,	  NULL,		TRUE},
 	{ NULL, NULL, NULL },
 };
 
 static const char * simple_help_screen =
 "Usage: cl_status <sub-command> [<options>] [<parameters>]\n"
 "\n"
-"The sub-commands usage is showed as below.\n"
-"Note: There is a general option -m for all subcommands, to make the output\n"
-"      more human readable. The default output should be easier for scripts\n"
-"      to parse.\n"
-"\n"
-"hbstatus\n"
-"	Indicate if heartbeat is running on the local system.\n"
-"\n"
-"listnodes\n"
-"	List the nodes in the cluster.\n"
-"	Options:\n"
-"	-p	list only 'ping' type nodes\n"
-"	-n	list only 'normal' type nodes\n"
-"\n"
-"nodestatus <node-name>\n"
-"	List the node status.\n"
-"\n"
-"nodetype <node-name>\n"
-"	List the nodes of a given type.\n"
-"       Curretly the available types are ping and normal.\n"
-"\n"
-"listhblinks <node-name>\n"
-"	List the network interfaces used as hearbeat links.\n"
-"\n"
-"hblinkstatus <node-name> <link-name>\n"
-"	Show the status of a heartbeat link\n"
-"\n"
+"Sub-commands:\n"
 "clientstatus <node-name> <client-id> [<timeout>]\n"
 "	Show the status of heartbeat clients.\n"
-"       For example, ping or ipfail.\n"
-"\n"
+"hblinkstatus <node-name> <link-name>\n"
+"	Show the status of a heartbeat link\n"
+"hbstatus\n"
+"	Indicate if heartbeat is running on the local system.\n"
+"listhblinks <node-name>\n"
+"	List the network interfaces used as hearbeat links.\n"
+"listnodes [<option>]\n"
+"	List the nodes in the cluster.\n"
+"	Options:\n"
+"		-p	list only 'ping' type nodes\n"
+"		-n	list only 'normal' type nodes\n"
+"nodestatus <node-name>\n"
+"	List the node status.\n"
+"nodetype <node-name>\n"
+"	List the nodes of a given type.\n"
 "rscstatus\n"
 "	Show the status of cluster resources.\n"
-"       Status will be one of: local, foreign, all or none.\n"
-"\n"
-"hbparamter -p <parameter-name>\n"
-"       Retrieve the value of cluster parameters.\n"
-"       The parameters may be one of the following, however some parameters\n"
-"       may not retrieved: hbversion, node, hopfudge, keepalive, deadtime,\n"
-"	deadping, warntime, initdead, watchdog, baud, udpport, logfacility,\n"
-"	logfile, debugfile, nice_failback, auto_failback, stonith,\n"
-"	stonith_host, respawn, rtprio, hbgenmethod, realtime, debug,\n"
-"	normalpoll, apiauth, msgfmt.\n\n"
-"	get the value of heartbeat parameters.\n"
-"	the parameters can be the one of the following, but maybe not all\n"
-"	can be actually gotten.\n"
-"	hbversion, node, hopfudge, keepalive, deadtime, deadping,\n"
-"	warntime, initdead, watchdog, baud, udpport, logfacility,\n"
-"	logfile, debugfile, nice_failback, auto_failback, stonith,\n"
-"	stonith_host, respawn, rtprio, hbgenmethod, realtime, debug,\n"
-"	normalpoll, apiauth, msgfmt\n"
-"\n";
+"hbparameter -p <parameter-name>\n"
+"	Retrieve the value of a cluster parameter.\n"
+"	As for the valid parameter names, please refer to the man page.\n";
 
-static const char * cl_status_name = "cl_status";
 static gboolean HB_SIGNON = FALSE;
 
+static const char * cl_status_name = "cl_status";
 /*
  * The following is to avoid cl_status sleeping forever. This is due to the 
  * hearbeat's abnormal status or even its crash.  
@@ -261,8 +235,8 @@ main(int argc, char ** argv)
 	int i = -1;
 	gboolean GOOD_CMD = FALSE;
 
-	if ((argc == 1) || ( argc == 2 && strncmp(argv[1],"-h",3) == 0)){
-		printf("%s",simple_help_screen);
+	if ((argc == 1) || ( argc == 2 && STRNCMP_CONST(argv[1],"-h") == 0)){
+		printf("%s", simple_help_screen);
 		return 0;
 	}
 
@@ -296,11 +270,18 @@ main(int argc, char ** argv)
 				return UNKNOWN_ERROR;
 			}
 
-			if (hb->llc_ops->signon(hb, cl_status_name)!= HA_OK) {
-				cl_log(LOG_ERR, "Cannot signon with heartbeat");
-				cl_log(LOG_ERR, "REASON: %s", 
-					hb->llc_ops->errmsg(hb));
+			/* cl_log(LOG_DEBUG, "Signing in with heartbeat.");*/
+			if (hb->llc_ops->signon(hb, NULL)!= HA_OK) {
 				ret_value = 1;
+				HB_SIGNON = FALSE;
+				if (cmds[i].needsignon) {
+					cl_log(LOG_ERR
+					,	"Cannot signon with heartbeat");
+					cl_log(LOG_ERR
+					,	"REASON: %s"
+					,	hb->llc_ops->errmsg(hb));
+					break;
+				}
 			}else{
 				HB_SIGNON = TRUE;
 			}
@@ -310,14 +291,14 @@ main(int argc, char ** argv)
 		}
 	}
 	if (GOOD_CMD == FALSE) {
-		cl_log(LOG_ERR, "%s is not a correct sub-command.", argv[1]);
+		cl_log(LOG_ERR, "%s: invalid sub-command.", argv[1]);
 		ret_value = PARAMETER_ERROR;
 	}
 
 	if (HB_SIGNON == TRUE) {
 		if (hb->llc_ops->signoff(hb) != HA_OK) {
 			cl_log(LOG_ERR, "Cannot sign off from heartbeat.");
-			cl_log(LOG_ERR, "REASON: %s\n", hb->llc_ops->errmsg(hb));
+			cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
 			/* Comment it to avoid to mask the subcommand's return
 			 * ret_value = UNKNOWN_ERROR;
 			 */
@@ -381,16 +362,16 @@ listnodes(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 				break;
 
 			default:
-				cl_log(LOG_ERR, "Error:getopt returned" 
+				cl_log(LOG_ERR, "Error: getopt returned" 
 					"character code %c.", option_char);
 				return PARAMETER_ERROR;
 		}
 	} while (1);
 
 	if (hb->llc_ops->init_nodewalk(hb) != HA_OK) {
-			cl_log(LOG_ERR, "Cannot start node walk.");
-			cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
-			return UNKNOWN_ERROR;
+		cl_log(LOG_ERR, "Cannot start node walk.");
+		cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
+		return UNKNOWN_ERROR;
 	}
 
 	if ( (ONLY_LIST_NORAMAL == TRUE) && (ONLY_LIST_PING == TRUE) ) {
@@ -405,10 +386,10 @@ listnodes(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 		} else {
 			type = hb->llc_ops->node_type(hb, node);
 			if (( ONLY_LIST_NORAMAL == TRUE )
-			      && (strncmp(type, "normal", 8)==0)) {
+			      && (STRNCMP_CONST(type, "normal")==0)) {
 				printf("%s\n", node);
 			} else if (( ONLY_LIST_PING == TRUE ) 
-				    && (strncmp(type, "ping", 8)==0)) {
+				    && (STRNCMP_CONST(type, "ping")==0)) {
 				printf("%s\n", node);
 			}
 		} 
@@ -428,8 +409,8 @@ listnodes(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 static int 
 nodestatus(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 {
-	const char * status;
-	int ret = UNKNOWN_ERROR;
+	const char *	status;
+	int		ret = UNKNOWN_ERROR;
 
 	if ( general_simple_opt_deal(argc, argv, optstr) < 0 ) {
 		/* There are option errors */
@@ -441,26 +422,29 @@ nodestatus(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 		return PARAMETER_ERROR;
 	}
 
+	cl_log(LOG_DEBUG, "optind: %d   argv[optindex+1]: %s", optind, 
+		argv[optind+1]);
 	status = hb->llc_ops->node_status(hb, argv[optind+1]);
 	if ( status == NULL ) {
 		fprintf(stderr, "Error. May be due to incorrect node name\n");
 		return PARAMETER_ERROR;
 	}
 	if (FOR_HUMAN_READ == TRUE) {
-		printf("The cluster nodes %s is %s\n", argv[optind+1], status);
+		printf("The cluster node %s is %s\n", argv[optind+1], status);
 	} else {
 		printf("%s\n", status);
 	}
 
-       if ( strncmp(status, "active", 6) == 0 
-       ||      strncmp(status, "up", 3) == 0) {
+       if ( STRNCMP_CONST(status, "active") == 0 
+       ||      STRNCMP_CONST(status, "up") == 0
+       ||      STRNCMP_CONST(status, "ping") == 0) {
 		ret = OK;   /* the node is active */
 	} else {
 		ret = NORMAL_FAIL;  /* the status = "dead" */
 	}
 	/* Should there be other status? According the comment in heartbeat.c
-	   three status: active, up, down. But it's not like that. 
-	*/
+	 * three status: active, up, down. But it's not like that. 
+	 */
 	
 	return ret;
 }
@@ -483,6 +467,8 @@ nodetype(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 		return PARAMETER_ERROR;
 	}
 
+	cl_log(LOG_DEBUG, "optind: %d   argv[optindex+1]: %s", optind, 
+		argv[optind+1]);
 	type = hb->llc_ops->node_type(hb, argv[optind+1]);
 	if ( type == NULL ) {
 		fprintf(stderr, "Error. May be due to incorrect node name\n");
@@ -512,6 +498,9 @@ listhblinks(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 		return PARAMETER_ERROR;
 	}
 
+	cl_log(LOG_DEBUG, "optind: %d   argv[optindex+1]: %s", optind, 
+		argv[optind+1]);
+
 	if (hb->llc_ops->init_ifwalk(hb, argv[optind+1]) != HA_OK) {
 		cl_log(LOG_ERR, "Cannot start heartbeat link interface walk.");
 		cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
@@ -552,7 +541,7 @@ hblinkstatus(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 	}
 
 	if_status = hb->llc_ops->if_status(hb, argv[optind+1], argv[optind+2]);
-	if (if_status == NULL) { /* Shoud be error ? */
+	if (if_status == NULL) { /* Should be error ? */
 		cl_log(LOG_ERR, "Cannot get heartbeat link status");
 		cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
 		return UNKNOWN_ERROR;
@@ -565,7 +554,7 @@ hblinkstatus(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 		printf("%s\n", if_status);
 	}
 
-	if ( strncmp(if_status, "up", 3) == 0 ) {
+	if ( STRNCMP_CONST(if_status, "up") == 0 ) {
 		ret = OK; /* the link is up */
 	} else {
 		ret = NORMAL_FAIL; /* the link is dead */
@@ -607,13 +596,13 @@ clientstatus(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 		printf("%s\n", cstatus);
 	}
 
-	if ( strncmp(cstatus, "online", 6) == 0 ) {
+	if ( STRNCMP_CONST(cstatus, "online") == 0 ) {
 		ret = OK; 
-	} else if ( strncmp(cstatus, "offline", 7) == 0 ) {
+	} else if ( STRNCMP_CONST(cstatus, "offline") == 0 ) {
 		ret = NORMAL_FAIL; /* the client is offline */
-	} else if ( strncmp(cstatus, "join", 4) == 0 ) {
+	} else if ( STRNCMP_CONST(cstatus, "join") == 0 ) {
 		ret = 2;
-	} else if ( strncmp(cstatus, "leave", 5) == 0 ) {
+	} else if ( STRNCMP_CONST(cstatus, "leave") == 0 ) {
 		ret = 3;
 	}
 	
@@ -632,7 +621,9 @@ rscstatus(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 
 	rstatus = hb->llc_ops->get_resources(hb);
 	if ( rstatus == NULL ) {
-		cl_log(LOG_ERR, "Cannot get client %s's status", argv[optind+2]);
+		cl_log(LOG_ERR
+		,	"Cannot get client %s's resource status"
+		,	argv[optind+2]);
 		cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
 		return UNKNOWN_ERROR;
 	}
@@ -647,9 +638,11 @@ rscstatus(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 static int
 hbparameter(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 {
-	int option_char;
-	int ret_value = 0;
-	gchar * paramname = NULL;
+	int		option_char;
+	int		ret_value = 0;
+	const char*	paramname = NULL;
+	char *		pvalue;
+
 	do {
 		option_char = getopt(argc, argv, optstr);
 
@@ -664,46 +657,40 @@ hbparameter(ll_cluster_t *hb, int argc, char ** argv, const char * optstr)
 
 			case 'p':
 				if (optarg) {
-					paramname = g_strdup(optarg);
-					cl_log(LOG_ERR,"parameter: %s", 
-						paramname);
+					paramname = optarg;
 				}
 				break;
 
 			default:
-				cl_log(LOG_ERR, "Error:getopt returned" 
+				cl_log(LOG_ERR, "Error: getopt returned" 
 					"character code %c.", option_char);
-				if (paramname != NULL) {
-					g_free(paramname);
-				}
 				return PARAMETER_ERROR;
 		}
 	} while (1);
 
-	if ( paramname != NULL ) {
-		char * pvalue;
-
-		pvalue = hb->llc_ops->get_parameter(hb, paramname);
-		if (pvalue == NULL) {
-			cl_log(LOG_ERR, "Cannot get parameter %s's value", 
-				argv[optind+2]);
-			cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
-			ret_value = NORMAL_FAIL;
-		} else {
-			if (FOR_HUMAN_READ == TRUE) {
-				printf("Heartbeat parameter %s's value: %s\n",
-					 paramname, pvalue);
-			} else {
-				printf("%s\n",pvalue);
-			}
-			free(pvalue);
-		}
-		g_free(paramname);
-	} else {
-		cl_log(LOG_ERR, "Missing parameter name");
-		ret_value = PARAMETER_ERROR;
+	if ( NULL == paramname) {
+		cl_log(LOG_ERR, "parameter name required");
+		return PARAMETER_ERROR;
 	}
 
+	pvalue = hb->llc_ops->get_parameter(hb, paramname);
+
+	if (pvalue == NULL) {
+		cl_log(LOG_ERR, "Cannot get parameter %s's value"
+		,	paramname);
+		cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
+		return NORMAL_FAIL;
+	}
+
+	if (FOR_HUMAN_READ == TRUE) {
+		printf("Heartbeat parameter %s's value: %s\n",
+			 paramname, pvalue);
+	} else {
+		printf("%s\n",pvalue);
+	}
+	cl_free(pvalue);
+	pvalue = NULL;
+
 	return ret_value;
 }
 
@@ -724,7 +711,7 @@ general_simple_opt_deal(int argc, char ** argv, const char * optstr)
 				break;
 
 			default:
-				cl_log(LOG_ERR, "Error:getopt returned"
+				cl_log(LOG_ERR, "Error: getopt returned"
 					"character code %c.", option_char);
 				return PARAMETER_ERROR;
 		}
