@@ -1,4 +1,4 @@
-/* $Id: config.c,v 1.113.2.7 2004/09/14 22:34:12 alan Exp $ */
+/* $Id: config.c,v 1.113.2.12 2005/11/14 20:55:29 gshi Exp $ */
 /*
  * Parse various heartbeat configuration files...
  *
@@ -54,11 +54,13 @@
 #include <pils/plugin.h>
 #include <clplumbing/realtime.h>
 #include <clplumbing/netstring.h>
+#include <clplumbing/coredumps.h>
 #include <HBcomm.h>
 #include <hb_module.h>
 #include <hb_api.h>
 #include <hb_config.h>
 #include <hb_api_core.h>
+#include <clplumbing/cl_misc.h>
 
 #define	DIRTYALIASKLUDGE
 
@@ -90,6 +92,8 @@ static int set_api_authorization(const char *);
 static int set_msgfmt(const char*);
 static int set_register_to_apphbd(const char *);
 static int set_badpack_warn(const char*);
+static int set_coredump(const char*);
+static int set_corerootdir(const char*);
 
 /*
  * Each of these parameters is is automatically recorded by
@@ -127,6 +131,8 @@ struct directive {
 , {KEY_MSGFMT,    set_msgfmt, TRUE, "classic", "message format in the wire"}
 , {KEY_REGAPPHBD, set_register_to_apphbd, FALSE, NULL, "register with apphbd"}
 , {KEY_BADPACK,   set_badpack_warn, TRUE, "true", "warn about bad packets"}
+, {KEY_COREDUMP,  set_coredump, TRUE, "true", "enable Linux-HA core dumps"}
+, {KEY_COREROOTDIR,set_corerootdir, TRUE, NULL, "set root directory of core dump area"}
 };
 
 static const struct WholeLineDirective {
@@ -145,7 +151,6 @@ extern int				parse_only;
 extern struct hb_media*			sysmedia[MAXMEDIA];
 extern struct sys_config *		config;
 extern struct sys_config  		config_init_value;
-extern int				verbose;
 extern volatile struct pstat_shm *	procinfo;
 extern volatile struct process_info *	curproc;
 extern char *				watchdogdev;
@@ -179,28 +184,6 @@ struct hb_media_fns**	hbmedia_types;
 	void setenv(const char *name, const char * value, int);
 #endif
 
-static int
-str_to_boolean(const char * s, int * ret)
-{
-	if (	strcasecmp(s, "true") == 0
-	||	strcasecmp(s, "on") == 0
-	||	strcasecmp(s, "yes") == 0
-	||	strcasecmp(s, "y") == 0
-	||	strcasecmp(s, "1") == 0){
-		*ret = TRUE;
-		return HA_OK;
-	}
-	if (	strcasecmp(s, "false") == 0
-	||	strcasecmp(s, "off") == 0
-	||	strcasecmp(s, "no") == 0
-	||	strcasecmp(s, "n") == 0
-	||	strcasecmp(s, "0") == 0){
-		*ret = FALSE;
-		return HA_OK;
-	}
-	return HA_FAIL;
-}
-
 /*
  *	Read in and validate the configuration file.
  *	Act accordingly.
@@ -462,6 +445,7 @@ parse_config(const char * cfgfile, char *nodename)
 	,	{"ccm",		"uid=" HA_CCMUSER}
 	,	{"ping",	"gid=" HA_APIGROUP}
 	,	{"cl_status",	"gid=" HA_APIGROUP}
+        ,       {"anon",        "gid=" HA_APIGROUP}
 	};
 
 	if ((f = fopen(cfgfile, "r")) == NULL) {
@@ -1209,6 +1193,7 @@ set_facility(const char * value)
 			return(HA_OK);
 		}
 	}
+     	ha_log(LOG_ERR, "facility(%s) not valid", value);
 	return(HA_FAIL);
 }
 
@@ -2014,11 +1999,49 @@ baddirective:
 
 }
 
+static int
+set_coredump(const char* value)
+{
+	gboolean	docore;
+	int		rc;
+	if ((rc = str_to_boolean(value, &docore)) == HA_OK) {
+		if (cl_enable_coredumps(docore) < 0 ) {
+			rc = HA_FAIL;
+		}
+	}
+	return rc;
+}
+
+static int
+set_corerootdir(const char* value)
+{
+	if (cl_set_corerootdir(value) < 0) {
+		cl_perror("Invalid core directory [%s]", value);
+		return HA_FAIL;
+	}
+	return HA_OK;
+}
 
 
 
 /*
  * $Log: config.c,v $
+ * Revision 1.113.2.12  2005/11/14 20:55:29  gshi
+ * make heartbeat accept a casual client from a user whose group id is haclient
+ *
+ * Revision 1.113.2.11  2005/08/11 21:08:23  gshi
+ * Move str_to_boolean function to clplumbing/cl_misc.c
+ * We use that function in ipfail.c to determine the value of auto_failback
+ *
+ * Revision 1.113.2.10  2005/08/05 15:50:41  gshi
+ * print out an error message if setting facility failed
+ *
+ * Revision 1.113.2.9  2005/06/18 13:01:53  alan
+ * Removed a declaration.
+ *
+ * Revision 1.113.2.8  2004/10/16 05:22:45  alan
+ * Put in new code to drop core files under /var/lib/heartbeat/cores/...
+ *
  * Revision 1.113.2.7  2004/09/14 22:34:12  alan
  * Changed cl_status from setuid to setgid
  *
