@@ -1,4 +1,4 @@
-/* $Id: findif.c,v 1.32.2.4 2004/09/11 20:52:32 alan Exp $ */
+/* $Id: findif.c,v 1.32.2.5 2004/12/15 16:43:49 alan Exp $ */
 /*
  * findif.c:	Finds an interface which can route a given address
  *
@@ -63,9 +63,6 @@
  */
 
 #include <portability.h>
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
 
 #include <stdio.h>
 #include <limits.h>
@@ -702,6 +699,9 @@ ff02::%lo0/32                     fe80::1%lo0                   UC          lo0
 
 /* 
  * $Log: findif.c,v $
+ * Revision 1.32.2.5  2004/12/15 16:43:49  alan
+ * Put in a number of portability fixes - most by Roberto Nibali (ratz).
+ *
  * Revision 1.32.2.4  2004/09/11 20:52:32  alan
  * Brought back changes from HEAD
  * 1) Added apphbd option
