@@ -1,4 +1,4 @@
-/* $Id: ha_msg_internal.c,v 1.39.2.2 2004/08/05 18:30:46 alan Exp $ */
+/* $Id: ha_msg_internal.c,v 1.39.2.3 2005/08/03 22:45:51 gshi Exp $ */
 /*
  * ha_msg_internal: heartbeat internal messaging functions
  *
@@ -224,6 +224,9 @@ isauthentic(const struct ha_msg * m)
 			ha_log(LOG_WARNING
 			,	"Bad/invalid auth token, authtoken=%p"
 			,	authtoken);
+			if (ANYDEBUG){
+				cl_log_message(m);
+			}
 		}
 		return(0);
 	}
@@ -357,6 +360,9 @@ main(int argc, char ** argv)
 #endif
 /*
  * $Log: ha_msg_internal.c,v $
+ * Revision 1.39.2.3  2005/08/03 22:45:51  gshi
+ * print out the ha message if authentication failed
+ *
  * Revision 1.39.2.2  2004/08/05 18:30:46  alan
  * Put in changes to support a new option which allows one to suppress
  * notification of packet corruption.
