@@ -1,4 +1,4 @@
-/* $Id: hb_api.c,v 1.94.2.5 2005/01/03 20:56:08 gshi Exp $ */
+/* $Id: hb_api.c,v 1.94.2.6 2005/11/14 20:55:29 gshi Exp $ */
 /*
  * hb_api: Server-side heartbeat API code
  *
@@ -1313,9 +1313,15 @@ api_check_client_authorization(client_proc_t* client)
 	 * then default to the "default" authorization category.
 	 * otherwise, use the client type's authorization list
 	 */
-	if (client->iscasual
-	||	(gauth = g_hash_table_lookup(APIAuthorization
-	,	client->client_id))	==  NULL) {
+	if (client->iscasual){
+		gauth = g_hash_table_lookup(APIAuthorization, "anon");
+		if (gauth == NULL){
+			cl_log(LOG_ERR, "NO auth found for anonymous");
+			return FALSE;
+		}
+		
+	}else if((gauth = g_hash_table_lookup(APIAuthorization,
+					client->client_id)) ==  NULL) {
 		if ((gauth = g_hash_table_lookup(APIAuthorization, "default"))
 		== NULL) {
 			client->removereason = "no default client auth";
