@@ -1,4 +1,4 @@
-/* $Id: hb_resource.c,v 1.48.2.11 2004/09/11 20:52:33 alan Exp $ */
+/* $Id: hb_resource.c,v 1.48.2.18 2005/10/27 17:32:24 blaschke Exp $ */
 /*
  * hb_resource: Linux-HA heartbeat resource management code
  *
@@ -149,7 +149,7 @@ void	Initiate_Reset(Stonith* s, const char * nodename, gboolean doreset);
 static int FilterNotifications(const char * msgtype);
 static int countbystatus(const char * status, int matchornot);
 static gboolean hb_rsc_isstable(void);
-static void shutdown_if_needed(void);
+static void PerformAutoFailback(void);
 
 ProcTrack_ops hb_rsc_RscMgmtProcessTrackOps = {
 	RscMgmtProcessDied,
@@ -282,22 +282,31 @@ FilterNotifications(const char * msgtype)
 	return rc;
 }
 
+static gboolean
+AutoFailbackProc(gpointer dummy)
+{
+	PerformAutoFailback();
+	return FALSE;
+}
+
 static void
 PerformAutoFailback(void)
 {
 	if (ANYDEBUG) {
 		cl_log(LOG_DEBUG, "Calling PerformAutoFailback()");
 	}
-	if ((procinfo->i_hold_resources & HB_FOREIGN_RSC) == 0
+	if (shutdown_in_progress
+	||	(procinfo->i_hold_resources & HB_FOREIGN_RSC) == 0
 	||	!rsc_needs_failback || !auto_failback) {
 		rsc_needs_failback = FALSE;
-		shutdown_if_needed();
+		hb_shutdown_if_needed();
 		return;
 	}
 
-	if (going_standby != NOT) {
-		cl_log(LOG_ERR, "Auto failback ignored.");
-		rsc_needs_failback = FALSE;
+	if (going_standby != NOT 
+	||	!other_is_stable || resourcestate != HB_R_STABLE) {
+		cl_log(LOG_DEBUG, "Auto failback delayed.");
+		Gmain_timeout_add(1*1000, AutoFailbackProc, NULL);
 		return;
 	}
 	if (ANYDEBUG) {
@@ -445,9 +454,55 @@ void
 hb_rsc_recover_dead_resources(struct node_info* hip)
 {
 	gboolean	need_stonith = TRUE;
-	standby_running = zero_longclock;
-	going_standby	= NOT;
+	struct ha_msg *	hmsg;
+	char		timestamp[16];
+
 	
+	if ((hmsg = ha_msg_new(6)) == NULL) {
+		cl_log(LOG_ERR, "no memory to takeover_from_node");
+		return;
+	}
+	
+	sprintf(timestamp, TIME_X, (TIME_T) time(NULL));
+
+	if (	ha_msg_add(hmsg, F_TYPE, T_STATUS) != HA_OK
+	||	ha_msg_add(hmsg, F_SEQ, "1") != HA_OK
+	||	ha_msg_add(hmsg, F_TIME, timestamp) != HA_OK
+	||	ha_msg_add(hmsg, F_ORIG, hip->nodename) != HA_OK
+	||	ha_msg_add(hmsg, F_STATUS, DEADSTATUS) != HA_OK) {
+		cl_log(LOG_ERR, "no memory to takeover_from_node");
+		ha_msg_del(hmsg);
+		return;
+	}
+	
+	if (hip->nodetype == PINGNODE_I) {
+		if (ha_msg_add(hmsg, F_COMMENT, "ping") != HA_OK) {
+			cl_log(LOG_ERR, "no memory to mark ping node dead");
+			ha_msg_del(hmsg);
+			return;
+		}
+	}
+	else if (going_standby != NOT) {
+		cl_log(LOG_INFO, "Cancelling pending standby operation");
+		going_standby = NOT;
+		standby_running = zero_longclock;
+
+		if ((!other_is_stable)
+		&&	((procinfo->i_hold_resources & HB_ALL_RSC) == HB_ALL_RSC)) {
+			other_is_stable = TRUE;
+			if (ANYDEBUG) {
+				cl_log(LOG_DEBUG
+				,	"hb_rsc_recover_dead_resources:"
+				" other now stable");
+			}
+		}
+		hb_shutdown_if_needed();
+	}
+	
+	/*deliver this message to clients*/
+	heartbeat_monitor(hmsg, KEEPIT, "<internal>");
+	ha_msg_del(hmsg);
+	hmsg = NULL;
 
 	if (hip->nodetype == PINGNODE_I) {
 		takeover_from_node(hip->nodename);
@@ -512,6 +567,15 @@ hb_rsc_recover_dead_resources(struct node_info* hip)
 		,	hip->nodename);
 		send_stonith_msg(hip->nodename, T_STONITH_UNNEEDED);
 		if (nice_failback) {
+			if ((procinfo->i_hold_resources & HB_ALL_RSC) == HB_ALL_RSC) {
+				other_is_stable = TRUE;
+				if (ANYDEBUG) {
+					cl_log(LOG_DEBUG
+					,	"hb_rsc_recover_dead_resources:"
+					" other now stable");
+				}
+				return;
+			}
 			/* These might happen due to timing weirdnesses */
 			if (! (procinfo->i_hold_resources & HB_LOCAL_RSC)){
 				req_our_resources(TRUE);
@@ -710,7 +774,7 @@ process_resources(const char * type, struct ha_msg* msg
 	enum hb_rsc_state	newrstate = resourcestate;
 	static int			first_time = 1;
 
-	shutdown_if_needed();
+	hb_shutdown_if_needed();
 	if (!DoManageResources || !nice_failback) {
 		return;
 	}
@@ -987,7 +1051,7 @@ process_resources(const char * type, struct ha_msg* msg
 	}
 
 	AuditResources();
-	shutdown_if_needed();
+	hb_shutdown_if_needed();
 }
 
 void
@@ -1148,13 +1212,14 @@ takeover_from_node(const char * nodename)
 	struct ha_msg *	hmsg;
 	char		timestamp[16];
 
+
 	if (hip == 0) {
 		return;
 	}
 	if (shutdown_in_progress) {
 		cl_log(LOG_INFO
 		,	"Resource takeover cancelled - shutdown in progress.");
-		shutdown_if_needed();
+		hb_shutdown_if_needed();
 		return;
 	}else if (hip->nodetype != PINGNODE_I) {
 		cl_log(LOG_INFO
@@ -1187,8 +1252,8 @@ takeover_from_node(const char * nodename)
 	}
 
 	/* Sending this message triggers the "mach_down" script */
-
-	heartbeat_monitor(hmsg, KEEPIT, "<internal>");
+	
+	/*heartbeat_monitor(hmsg, KEEPIT, "<internal>");*/
 	QueueRemoteRscReq(PerformQueuedNotifyWorld, hmsg);
 
 	/*
@@ -1662,7 +1727,7 @@ ask_for_resources(struct ha_msg *msg)
 	if (ANYDEBUG) {
 		cl_log(LOG_INFO, "New standby state: %d", going_standby);
 	}
-	shutdown_if_needed();
+	hb_shutdown_if_needed();
 }
 
 static int
@@ -1831,8 +1896,8 @@ go_standby(enum standby who, int resourceset) /* Which resources to give up */
 
 }
 
-static void
-shutdown_if_needed(void)
+void
+hb_shutdown_if_needed(void)
 {
 	if (rsc_needs_shutdown) {
 		hb_giveup_resources();
@@ -1866,7 +1931,7 @@ hb_giveup_resources(void)
 
 	if (!hb_rsc_isstable()) {
 		/* Try again later... */
-		/* (through shutdown_if_needed()) */
+		/* (through hb_shutdown_if_needed()) */
 		if (!rsc_needs_shutdown) {
 			cl_log(LOG_WARNING
 			,	"Shutdown delayed until current"
@@ -2113,7 +2178,7 @@ RscMgmtProcessDied(ProcTrack* p, int status, int signo, int exitcode
 
 	p->privatedata = NULL;
 	StartNextRemoteRscReq();
-	shutdown_if_needed();
+	hb_shutdown_if_needed();
 }
 
 static const char *
@@ -2225,7 +2290,7 @@ StartNextRemoteRscReq(void)
 	hook = g_hook_first_valid(&RemoteRscReqQueue, FALSE);
 	if (hook == NULL) {
 		ResourceMgmt_child_count = 0;
-		shutdown_if_needed();
+		hb_shutdown_if_needed();
 		return;
 	}
 
@@ -2284,6 +2349,9 @@ StonithProcessDied(ProcTrack* p, int status, int signo, int exitcode, int waslog
 		,	h->nodename);
 
 		Gmain_timeout_add(5*1000, StonithProc, h);
+		/* Don't free 'h' - we still need it */
+		p->privatedata = NULL;
+		return;
 	}else{
 		/* We need to finish taking over the other side's resources */
 		takeover_from_node(h->nodename);
@@ -2332,6 +2400,36 @@ StonithStatProcessName(ProcTrack* p)
 
 /*
  * $Log: hb_resource.c,v $
+ * Revision 1.48.2.18  2005/10/27 17:32:24  blaschke
+ *
+ * Bug 927 - Shutdown if needed after standby is cancelled.
+ *
+ * Revision 1.48.2.17  2005/10/26 15:34:09  blaschke
+ *
+ * Bug 835 - Instead of ignoring auto failback if going standby, delay auto
+ * failback one second if going standby or either node is unstable
+ *
+ * Revision 1.48.2.16  2005/10/11 15:38:29  blaschke
+ *
+ * Bug 559 Part II - Don't cancel standby op because ping node died
+ *
+ * Revision 1.48.2.15  2005/10/10 17:12:19  blaschke
+ *
+ * Bug 559 - Stop nodes waiting for other to become stable even after other
+ * is dead.
+ *
+ * Revision 1.48.2.14  2005/08/30 17:45:59  davidlee
+ * Declarations must precede statements
+ *
+ * Revision 1.48.2.13  2004/11/11 22:13:53  gshi
+ *
+ * de dead notice should be called after the node is really decared dead
+ * instead of after receving a shutdown message from it
+ *
+ * Revision 1.48.2.12  2004/10/20 15:29:25  alan
+ * Fixed a bug related to repeat STONITHs failing because we freed some
+ * data we shouldn't have.
+ *
  * Revision 1.48.2.11  2004/09/11 20:52:33  alan
  * Brought back changes from HEAD
  * 1) Added apphbd option
