@@ -1,4 +1,4 @@
-/* $Id: ipc.h,v 1.30.2.1 2004/04/20 07:57:54 alan Exp $ */
+/* $Id: ipc.h,v 1.30.2.2 2004/12/08 19:14:47 gshi Exp $ */
 /*
  * ipc.h IPC abstraction data structures.
  *
@@ -125,6 +125,20 @@ struct IPC_CHANNEL{
 	void*		ch_private;	/* channel private data. */
 					/* (may contain conn. info.) */
 	IPC_Ops*	ops;		/* IPC_Channel function table.*/
+	
+	
+	/* the number of bytes remainng to send for the first message in send queue
+           0 means nothing has been sent thus all bytes needs to be send
+           n != 0 means there are still n bytes needs to be sent
+        */
+        unsigned int    bytes_remaining;
+	
+	
+        /* is the send blocking or nonblocking*/
+        gboolean        is_send_blocking;
+	
+
+
 /*
  *  There are two queues in channel. One is for sending and the other
  *  is for receiving. 
