@@ -1,3 +1,4 @@
+/* $Id: realtime.h,v 1.5.2.1 2004/04/20 07:57:54 alan Exp $ */
 #ifndef _CLPLUMBING_REALTIME_H
 #	define _CLPLUMBING_REALTIME_H
 #	include <sched.h>
@@ -18,6 +19,7 @@
 void cl_make_realtime
 (	int spolicy,	/* SCHED_RR or SCHED_FIFO */
 	int priority,	/* typically 1-99 */
+	int stackgrowK,	/* Amount to grow stack by */
 	int heapgrowK	/* Amount to grow heap by */
 );
 
@@ -33,4 +35,10 @@ void cl_enable_realtime(void);
 /* Sleep a really short (the shortest) time */
 int cl_shortsleep(void);
 
+/* Print messages if we've done (more) non-realtime mallocs */
+void cl_realtime_malloc_check(void);
+
+/* Number of times we "go to the well" for memory after becoming realtime */
+int cl_nonrealtime_malloc_count(void);
+
 #endif
