@@ -1,4 +1,4 @@
-/* $Id: ha_msg.h,v 1.24.2.1 2004/04/20 07:57:54 alan Exp $ */
+/* $Id: ha_msg.h,v 1.24.2.2 2004/09/22 17:01:09 gshi Exp $ */
 /*
  * Intracluster message object (struct ha_msg)
  *
@@ -166,9 +166,20 @@ int		ha_msg_addbin(struct ha_msg * msg, const char * name,
 int		ha_msg_add(struct ha_msg * msg
 		,	const char* name, const char* value);
 
-/* Modify null-terminated name and a value to the message */
-int		ha_msg_mod(struct ha_msg * msg
-		,	const char* name, const char* value);
+int             cl_msg_modstring(struct ha_msg * msg,
+                           const char* name,
+                           const char* value);
+int             cl_msg_modbin(struct ha_msg * msg,
+                              const char* name, 
+                              const void* value,
+                              size_t vlen);
+int             cl_msg_modstruct(struct ha_msg * msg,
+                                 const char* name,
+                                 const struct ha_msg* value);
+
+#define ha_msg_mod(msg, name, value) cl_msg_modstring(msg, name, value)
+
+
 
 /* Add name, value (with known lengths) to the message */
 int		ha_msg_nadd(struct ha_msg * msg, const char * name, int namelen
