@@ -1,4 +1,4 @@
-/* $Id: base64.c,v 1.10.2.1 2004/04/20 08:39:45 alan Exp $ */
+/* $Id: base64.c,v 1.10.2.2 2005/01/12 02:25:46 horms Exp $ */
 #include <portability.h>
 #include <heartbeat.h>
 #include <syslog.h>
@@ -192,7 +192,7 @@ base64_to_binary(const char * in, int inlen, void * output, int outlen)
 
 	/* We have enough space.  We are happy :-)  */
 
-	startout = out = (char *)output;
+	startout = out = (unsigned char *)output;
 
 	while (input < lastinput) {
 		Char2SixBits(*input, sixbits1); ++input;
