@@ -1,4 +1,4 @@
-/* $Id: cl_malloc.c,v 1.1.2.3 2004/09/14 15:50:27 alan Exp $ */
+/* $Id: cl_malloc.c,v 1.1.2.4 2005/01/12 02:25:46 horms Exp $ */
 #include <portability.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -494,7 +494,7 @@ cl_dump_item(struct cl_bucket*b)
 	,	(long)(b->hdr.bucket >= NUMBUCKS ? 0 
 	:	cl_bucket_sizes[b->hdr.bucket]));
 #endif
-	cbeg = ((char *)b)+cl_malloc_hdr_offset;
+	cbeg = ((unsigned char *)b)+cl_malloc_hdr_offset;
 	cend = cbeg+b->hdr.reqsize+GUARDSIZE;
 
 	for (cp=cbeg; cp < cend; cp+= sizeof(unsigned)) {
