@@ -248,7 +248,7 @@ peel_netstring(const char * s, const char * smax, int* len,
 		sp ++;
 	}
 
-	if (sp >= smax ){
+	if (sp >= smax) {
 		return(HA_FAIL);
 	}
 
@@ -257,6 +257,9 @@ peel_netstring(const char * s, const char * smax, int* len,
 	*data = sp;
 
 	sp += (*len);
+	if (sp >= smax) {
+		return(HA_FAIL);
+	}
 	if (*sp != ','){
 		return(HA_FAIL);
 	}
@@ -412,7 +415,7 @@ is_auth_netstring(const char * datap, size_t datalen,
 	}
 
 
-	if (authmethod(authwhich, datap, datalen, authstr, authlen)
+	if (authmethod(authwhich, datap, datalen, authstr, MAXLINE)
 	!=	authwhich) {
 
 		if (!cl_msg_quiet_fmterr) {
