@@ -1,4 +1,4 @@
-/* $Id: ipcsocket.c,v 1.88.2.2 2004/09/17 05:34:39 alan Exp $ */
+/* $Id: ipcsocket.c,v 1.88.2.4 2005/01/12 02:25:46 horms Exp $ */
 /*
  * ipcsocket unix domain socket implementation of IPC abstraction.
  *
@@ -455,7 +455,7 @@ socket_accept_connection(struct IPC_WAIT_CONNECTION * wait_conn
 {
 	struct sockaddr_un			peer_addr;
 	struct IPC_CHANNEL *			ch;
-	int					sin_size;
+	socklen_t				sin_size;
 	int					s;
 	int					new_sock;
 	struct SOCKET_WAIT_CONN_PRIVATE*	conn_private;
@@ -630,22 +630,34 @@ socket_send(struct IPC_CHANNEL * ch, struct IPC_MESSAGE* msg)
 	if (msg->msg_len < 0 || msg->msg_len > MAXDATASIZE) {
 		return IPC_FAIL;
 	}
-  
-	if (ch->ch_status == IPC_CONNECT
-	&&	ch->send_queue->current_qlen < ch->send_queue->max_qlen) {
-		/* add the message into the send queue */
-		CHECKFOO(0,ch, msg, SavedQueuedBody, "queued message");
-		SocketIPCStats.noutqueued++;
-		ch->send_queue->queue = g_list_append(ch->send_queue->queue
-		,	msg);
-		ch->send_queue->current_qlen++;
-		/* resume io */
-		ch->ops->resume_io(ch);
-		return IPC_OK;
-	}
-  
-	return IPC_FAIL;
-  
+
+	if (ch->ch_status != IPC_CONNECT){
+                return IPC_FAIL;
+        }
+	
+	if ( !ch->is_send_blocking &&
+	     ch->send_queue->current_qlen >= ch->send_queue->max_qlen) {
+                cl_log(LOG_ERR, "send queue maximum length(%d) exceeded",
+                       ch->send_queue->max_qlen );
+                return IPC_FAIL;
+        }
+
+        do{
+                ch->ops->resume_io(ch);
+        }
+        while (ch->send_queue->current_qlen >= ch->send_queue->max_qlen);
+	
+        /* add the message into the send queue */
+        CHECKFOO(0,ch, msg, SavedQueuedBody, "queued message");
+        SocketIPCStats.noutqueued++;
+        ch->send_queue->queue = g_list_append(ch->send_queue->queue,
+                                              msg);
+        ch->send_queue->current_qlen++;
+        /* resume io */
+        ch->ops->resume_io(ch);
+        return IPC_OK;
+	
+	
 }
 
 static int 
@@ -1018,31 +1030,36 @@ socket_resume_io_read(struct IPC_CHANNEL *ch, gboolean* started, gboolean read1a
 	if (retcode != IPC_OK) {
 		return retcode;
 	}
-
+	
 	return IPC_ISRCONN(ch) ? IPC_OK : IPC_BROKEN;
 }
 
 static int
-socket_resume_io_write(struct IPC_CHANNEL *ch, gboolean* started)
-{
+socket_resume_io_write(struct IPC_CHANNEL *ch, int* nmsg){
+
 	int				retcode = IPC_OK;
 	struct SOCKET_CH_PRIVATE*	conn_info;
-
-
+		
 	CHANAUDIT(ch);
 	conn_info = (struct SOCKET_CH_PRIVATE *) ch->ch_private;
-	*started = FALSE;
+
+	*nmsg = 0;
   
  
 	while (ch->ch_status == IPC_CONNECT
 	&&		retcode == IPC_OK
 	&&		ch->send_queue->current_qlen > 0) {
-
+		
 		GList *				element;
 		struct IPC_MESSAGE *		msg;
 		struct SOCKET_MSG_HEAD		head;
 		int				sendrc = 0;
-
+		char*				p;
+		int				len;
+		int				head_len = sizeof(struct SOCKET_MSG_HEAD);
+		int				bytes_send;
+		int				i;
+		
 		CHANAUDIT(ch);
 		element = g_list_first(ch->send_queue->queue);
 		if (element == NULL) {
@@ -1052,108 +1069,104 @@ socket_resume_io_write(struct IPC_CHANNEL *ch, gboolean* started)
 		}
 		msg = (struct IPC_MESSAGE *) (element->data);
 		head.msg_len = msg->msg_len;
-
-		/* Send message header */
-		sendrc = send(conn_info->s, (char *)&head
-		,	sizeof(struct SOCKET_MSG_HEAD)
-		,	(MSG_DONTWAIT|MSG_NOSIGNAL));
-		SocketIPCStats.last_send_rc = sendrc;
-		SocketIPCStats.last_send_errno = errno;
-		++SocketIPCStats.send_count;
-#ifdef DEBUG
-		cl_log(LOG_DEBUG, "Sent %d byte message header"
-		,	sizeof(struct SOCKET_MSG_HEAD));
-#endif
-
-
-		if (sendrc < 0) {
-			switch (errno) {
-				case EAGAIN:
-#ifdef DEBUG
-					cl_log(LOG_DEBUG,
-						"socket send returned EAGAIN");
-#endif
-					/* FIXME! KLUDGE! */
-					/* We could fix this if we kept better
-					 * state info so we could retry this
-					 * operation later and not be confused.
-					 * This is the right thing to do!
-					 */
-					cl_shortsleep();
-					continue;
-				case EPIPE:
-					socket_check_disc_pending(ch);
-					retcode = IPC_BROKEN;
-					break;
-				default:
-					cl_perror("socket_resume_io_write: send1 bad errno");
-					ch->ch_status = IPC_DISCONNECT;
-					retcode = IPC_FAIL;
-					break;
-			}
-			break;
-    		}
-		*started=TRUE;
-
-
-		do {
-			CHANAUDIT(ch);
-			sendrc = send(conn_info->s, msg->msg_body
-			,	msg->msg_len, (MSG_DONTWAIT|MSG_NOSIGNAL));
-			SocketIPCStats.last_send_rc = sendrc;
-			SocketIPCStats.last_send_errno = errno;
-			++SocketIPCStats.send_count;
-#ifdef DEBUG
-			cl_log(LOG_DEBUG, "send(%d bytes)  => %d errno=%d"
-			,	msg->msg_len, sendrc, errno);
-#endif
-
-			/* if send failed with EAGAIN, delay and try again */
-			/* FIXME! KLUDGE! */
-			/* We could fix this if we kept better
-			 * state info so we could retry this
-			 * operation later and not be confused.
-			 * This is the right thing to do!
-			 */
-		} while(sendrc < 0
-		&&	(errno == EAGAIN ? (cl_shortsleep(), TRUE) : FALSE));
-
-		if (sendrc > 0 && sendrc != (int)msg->msg_len) {
-			cl_perror("Sent %d byte message body: rc = %d"
-			,	(int)msg->msg_len, sendrc);
+		
+		if (ch->bytes_remaining  == 0){
+			/*start to send a new message*/
+			bytes_send = 0;
+		}else {
+			bytes_send = head.msg_len + head_len - ch->bytes_remaining;
 		}
 
-#ifdef DEBUG
-		cl_log(LOG_DEBUG, "Sent %d byte message body"
-		,	msg->msg_len);
-		cl_log(LOG_DEBUG, "Contents sent: %s",(char*)msg->msg_body);
-#endif
+		if (bytes_send < 0){
+			cl_log(LOG_ERR, "socket_resume_io_write: wrong length"
+			       "head.msg_len =%d, head_len =%d, ch->bytes_remaining=%d",
+			       head.msg_len, head_len, ch->bytes_remaining);
+			return IPC_FAIL;
+		}
 
-		if (sendrc < 0) {
-			switch (errno) {
-				case EPIPE:
-					socket_check_disc_pending(ch);
-					retcode = IPC_BROKEN;
-					break;
-				default:
-					cl_perror("socket_resume_io_write"
-					": send2 bad errno");
-					ch->ch_status = IPC_DISCONNECT;
-					retcode = IPC_FAIL;
-					break;
+		
+		for ( i = 0 ; i < 2; i++){
+
+			switch (i){
+
+			case 0:
+				/* for sending out head*/
+				if (bytes_send <  head_len ){
+					/*we need send head or partial head*/
+					p= ((char*)&head) + bytes_send;
+					len = head_len - bytes_send;
+				}else{
+					/*head is already sent out*/
+					continue;
+				}
+				
+				break;
+			default:
+				/* for sending out the message body*/
+				p = (char*)msg->msg_body + bytes_send - head_len;
+				len = msg->msg_len + head_len - bytes_send;
+				
 			}
-			break;
-		}else{
-			CHECKFOO(3,ch, msg, SavedSentBody, "sent message")
-			ch->send_queue->queue = g_list_remove(
-					ch->send_queue->queue,	msg);
+			
+			do{
+				sendrc = send(conn_info->s, p, len, (MSG_DONTWAIT|MSG_NOSIGNAL));
+				SocketIPCStats.last_send_rc = sendrc;
+				SocketIPCStats.last_send_errno = errno;
+				++SocketIPCStats.send_count;
+				
+				if (sendrc < 0) {
+					switch (errno) {
+					case EAGAIN:
+						retcode = IPC_OK;
+						break;
+					case EPIPE:
+						socket_check_disc_pending(ch);
+						retcode = IPC_BROKEN;
+						break;
+					default:
+						cl_perror("socket_resume_io_write: send1 bad errno");
+						ch->ch_status = IPC_DISCONNECT;
+						retcode = IPC_FAIL;
+						break;
+					}
+					ch->bytes_remaining = msg->msg_len + head_len - bytes_send;	
+					goto out;
+				}
+				
+				p += sendrc;
+				len -= sendrc;
+				
+				bytes_send += sendrc;
+				
+			}while( len > 0);
+			
+		}
+		
+		if (bytes_send > head_len + msg->msg_len ){
+			cl_log(LOG_INFO, "socket_resme_io_write:"
+			       " sending out more bytes then the message has");
+			return IPC_FAIL;
+		}
+		
+		if (bytes_send == head_len + msg->msg_len ){
+			ch->send_queue->queue = 
+				g_list_remove(ch->send_queue->queue,	msg);
+			
 			if (msg->msg_done != NULL) {
 				msg->msg_done(msg);
 			}
 			SocketIPCStats.nsent++;
 			ch->send_queue->current_qlen--;
+			(*nmsg)++;
+			ch->bytes_remaining = 0;
+		}else {
+			ch->bytes_remaining = msg->msg_len + head_len - bytes_send;			
+			break;
 		}
 	}
+
+ out:
+
 	CHANAUDIT(ch);
 	if (retcode != IPC_OK) {
 		return retcode;
@@ -1164,10 +1177,12 @@ socket_resume_io_write(struct IPC_CHANNEL *ch, gboolean* started)
 static int
 socket_resume_io(struct IPC_CHANNEL *ch)
 {
-	int		rc1, rc2;
+	int		rc1 = IPC_OK;
+	int		rc2 = IPC_OK;
 	gboolean	rstarted;
 	gboolean	wstarted;
 	gboolean	OKonce = FALSE;
+	int		nwmsg = 1;
 #ifdef DEBUG
 	int		count = 0;
 #endif
@@ -1183,9 +1198,13 @@ socket_resume_io(struct IPC_CHANNEL *ch)
 		if (ch->ch_status == IPC_DISC_PENDING) {
 			rc2 = IPC_OK;
 			wstarted = FALSE;
-		}else{
-			rc2 = socket_resume_io_write(ch, &wstarted);
 		}
+		
+		if (nwmsg > 0){
+			nwmsg = 0;
+			rc2 = socket_resume_io_write(ch, &nwmsg);
+		}
+		
 		CHANAUDIT(ch);
 		if (rc1 == IPC_OK || rc2 == IPC_OK) {
 			OKonce = TRUE;
@@ -1198,7 +1217,7 @@ socket_resume_io(struct IPC_CHANNEL *ch)
 			,	rstarted, wstarted, count);
 		}
 #endif
-	} while ((rstarted||wstarted) && IPC_ISRCONN(ch));
+	} while ((rstarted||nwmsg > 0) && IPC_ISRCONN(ch));
 
 	if (IPC_ISRCONN(ch)) {
 		if (rc1 != IPC_OK) {
@@ -1473,6 +1492,8 @@ socket_client_channel_new(GHashTable *ch_attrs) {
 #endif
   temp_ch->ch_private = (void*) conn_info;
   temp_ch->ops = (struct IPC_OPS *)&socket_ops;
+  temp_ch->bytes_remaining = 0;
+  temp_ch->is_send_blocking = TRUE;
   temp_ch->send_queue = socket_queue_new();
   temp_ch->recv_queue = socket_queue_new();
    
@@ -1501,6 +1522,8 @@ socket_server_channel_new(int sockfd){
   temp_ch->ch_status = IPC_DISCONNECT;
   temp_ch->ch_private = (void*) conn_info;
   temp_ch->ops = (struct IPC_OPS *)&socket_ops;
+  temp_ch->bytes_remaining = 0;
+  temp_ch->is_send_blocking = TRUE;
   temp_ch->send_queue = socket_queue_new();
   temp_ch->recv_queue = socket_queue_new();
    
