@@ -1,4 +1,4 @@
-/* $Id: realtime.c,v 1.19.2.1 2004/04/20 08:39:45 alan Exp $ */
+/* $Id: realtime.c,v 1.19.2.2 2005/01/12 02:25:46 horms Exp $ */
 #include <portability.h>
 #include <sys/types.h>
 #include <stdlib.h>
@@ -48,7 +48,7 @@ cl_stack_hogger(char * inbuf, int kbytes)
 	}
 
 	if (kbytes > 0) {
-		return cl_stack_hogger(buf, kbytes-1);
+		return cl_stack_hogger((char*)buf, kbytes-1);
 	}else{
 		return buf[sizeof(buf)-1];
 	}
