@@ -1,4 +1,4 @@
-/* $Id: sha1.c,v 1.10.2.1 2004/04/20 09:08:50 lars Exp $ */
+/* $Id: sha1.c,v 1.10.2.2 2005/01/12 02:25:46 horms Exp $ */
 /*
 SHA-1 in C
 By Steve Reid <steve@edmweb.com>
@@ -288,9 +288,9 @@ sha1_auth_calc (const struct HBauth_info *info
 		return FALSE;
 	}
 
-	key = g_strdup(info->key);
+	key = (unsigned char*) g_strdup(info->key);
 
-	key_len = strlen(key);
+	key_len = strlen((char*)key);
 
 	if (key_len > SHA_BLOCKSIZE) {
 		SHA1_CTX         tctx ;
@@ -329,8 +329,8 @@ sha1_auth_calc (const struct HBauth_info *info
 
 	result[0] = '\0';
 	for (i = 0; i < SHA_DIGESTSIZE; i++) {
-		sprintf(tk, "%02x", osha[i]);
-		strcat(result, tk);
+		sprintf((char*)tk, "%02x", osha[i]);
+		strcat(result, (char*)tk);
 	}
 	g_free(key);
 
