@@ -1,4 +1,4 @@
-/* $Id: mcast.c,v 1.15.2.3 2004/05/12 00:10:14 alan Exp $ */
+/* $Id: mcast.c,v 1.15.2.4 2005/01/12 02:25:47 horms Exp $ */
 /*
  * mcast.c: implements hearbeat API for UDP multicast communication
  *
@@ -413,7 +413,7 @@ mcast_read(struct hb_media* hbm, int *lenp)
 {
 	struct mcast_private *	mcp;
 	char			buf[MAXLINE];
-	int			addr_len = sizeof(struct sockaddr);
+	socklen_t		addr_len = sizeof(struct sockaddr);
    	struct sockaddr_in	their_addr; /* connector's addr information */
 	int	numbytes;
 	void	*pkt;
@@ -831,6 +831,9 @@ get_loop(const char *loop, u_char *l)
 
 /*
  * $Log: mcast.c,v $
+ * Revision 1.15.2.4  2005/01/12 02:25:47  horms
+ * Fix gcc-4.0 compile problems (Andreas Jochens)
+ *
  * Revision 1.15.2.3  2004/05/12 00:10:14  alan
  * Changed all the HBcomm plugins to use PILCallLog() for logging
  * instead of calling the function pointer directly.
