@@ -1,4 +1,4 @@
-/* $Id: ping.c,v 1.26.2.3 2004/09/14 16:35:32 alan Exp $ */
+/* $Id: ping.c,v 1.26.2.6 2005/08/10 04:08:17 horms Exp $ */
 /*
  * ping.c: ICMP-echo-based heartbeat code for heartbeat.
  *
@@ -311,6 +311,9 @@ ping_close(struct hb_media* mp)
 
 /*
  * Receive a heartbeat ping reply packet.
+ * This code only needs to run once per instance of heartbeat.
+ * Otherwise we receive "n" copies of every single ping packet.
+ * FIXME!!
  */
 
 static void *
@@ -323,7 +326,7 @@ ping_read(struct hb_media* mp, int *lenp)
 	}buf;
 	const char *		bufmax = ((char *)&buf)+sizeof(buf);
 	char *			msgstart;
-	int			addr_len = sizeof(struct sockaddr);
+	socklen_t		addr_len = sizeof(struct sockaddr);
    	struct sockaddr_in	their_addr; /* connector's addr information */
 	struct ip *		ip;
 	struct icmp		icp;
@@ -336,6 +339,8 @@ ping_read(struct hb_media* mp, int *lenp)
 	
 	PINGASSERT(mp);
 	ei = (struct ping_private *) mp->pd;
+
+ReRead:	/* We recv lots of packets that aren't ours */
 	
 	if ((numbytes=recvfrom(ei->sock, (void *) &buf.cbuf
 	,	sizeof(buf.cbuf)-1, 0,	(struct sockaddr *)&their_addr
@@ -365,7 +370,7 @@ ping_read(struct hb_media* mp, int *lenp)
 	memcpy(&icp, (buf.cbuf + hlen), sizeof(icp));
 	
 	if (icp.icmp_type != ICMP_ECHOREPLY || icp.icmp_id != ei->ident) {
-		return NULL;
+		goto ReRead;	/* Not one of ours */
 	}
 
 	if (DEBUGPKT) {
@@ -380,6 +385,7 @@ ping_read(struct hb_media* mp, int *lenp)
 	
 	pktlen = numbytes - hlen - ICMP_HDR_SZ;
 	if ((pkt = ha_malloc(pktlen + 1)) == NULL) {
+		errno = ENOMEM;
 		return NULL;
 	}
 	pkt[pktlen] = 0;
@@ -391,21 +397,19 @@ ping_read(struct hb_media* mp, int *lenp)
 	msg = wirefmt2msg(msgstart, bufmax - msgstart);
 	if (msg == NULL) {
                 ha_free(pkt);
+		errno = EINVAL;
 		return(NULL);
 	}
 	comment = ha_msg_value(msg, F_COMMENT);
 	if (comment == NULL || strcmp(comment, PIL_PLUGIN_S)) {
                 ha_free(pkt);
 		ha_msg_del(msg);
+		errno = EINVAL;
 		return(NULL);
 	}
 	
 	ha_msg_del(msg);
-	/* return(msg); */
-	
 	return(pkt);
-
-
 }
 
 /*
