@@ -1,4 +1,4 @@
-/* $Id: ping_group.c,v 1.4.2.3 2004/06/16 07:39:27 horms Exp $ */
+/* $Id: ping_group.c,v 1.4.2.5 2005/08/10 04:08:17 horms Exp $ */
 /*
  * ping_group.c: ICMP-echo-based heartbeat code for heartbeat.
  *
@@ -412,7 +412,7 @@ ping_group_read(struct hb_media* mp, int *lenp)
 	}buf;
 	const char *		bufmax = ((char *)&buf)+sizeof(buf);
 	char *			msgstart;
-	int			addr_len = sizeof(struct sockaddr);
+	socklen_t		addr_len = sizeof(struct sockaddr);
    	struct sockaddr_in	their_addr; /* connector's addr information */
 	struct ip *		ip;
 	struct icmp		icp;
@@ -429,6 +429,8 @@ ping_group_read(struct hb_media* mp, int *lenp)
 	PINGGROUPASSERT(mp);
 	ei = (ping_group_private_t *) mp->pd;
 
+ReRead:        /* We recv lots of packets that aren't ours */
+
 	if ((numbytes=recvfrom(ei->sock, (void *) &buf.cbuf
 	,	sizeof(buf.cbuf)-1, 0,	(struct sockaddr *)&their_addr
 	,	&addr_len)) < 0) {
@@ -457,7 +459,7 @@ ping_group_read(struct hb_media* mp, int *lenp)
 	memcpy(&icp, (buf.cbuf + hlen), sizeof(icp));
 
 	if (icp.icmp_type != ICMP_ECHOREPLY || icp.icmp_id != ei->ident) {
-		return(NULL);
+		goto ReRead;	/* Not ours */
 	}
 	seq = ntohs(icp.icmp_seq);
 
@@ -475,7 +477,7 @@ ping_group_read(struct hb_media* mp, int *lenp)
 	for(node = ei->node; node; node = node->next) {
 		if(!memcmp(&(their_addr.sin_addr), &(node->addr.sin_addr),
 					sizeof(struct in_addr))) {
-			break;
+			goto ReRead;    /* Not ours */
 		}
 	}
 	if(!node) {
@@ -484,11 +486,13 @@ ping_group_read(struct hb_media* mp, int *lenp)
 
 	msg = wirefmt2msg(msgstart, bufmax - msgstart);
 	if(msg == NULL) {
+		errno = EINVAL;
 		return(NULL);
 	}
 	comment = ha_msg_value(msg, F_COMMENT);
 	if(comment == NULL || strcmp(comment, PIL_PLUGIN_S)) {
 		ha_msg_del(msg);
+		errno = EINVAL;
 		return(NULL);
 	}
 
@@ -496,13 +500,17 @@ ping_group_read(struct hb_media* mp, int *lenp)
 	if(ei->slot[slotn] == seq) {
 		/* Duplicate within window */
 		ha_msg_del(msg);
-		return(NULL);
+		goto ReRead;	/* Not ours */
 	}
 
 	ei->slot[slotn] = seq;
 	
 	pktlen = numbytes - hlen - ICMP_HDR_SZ;
-	pkt = ha_malloc(pktlen + 1);
+	if (NULL == (pkt = ha_malloc(pktlen + 1))) {
+		ha_msg_del(msg);
+		errno = ENOMEM;
+		return NULL;
+	}
 	pkt[pktlen] = 0;
 	
 	memcpy(pkt, buf.cbuf + hlen + ICMP_HDR_SZ, pktlen);	
