@@ -1,5 +1,4 @@
-const static char _serial_c_Id [] = "$Id: serial.c,v 1.24.2.1 2003/06/02 11:24:37 lars Exp $";
-
+/* $Id: serial.c,v 1.29.2.4 2004/05/12 02:04:38 alan Exp $ */
 /*
  * Linux-HA serial heartbeat code
  *
@@ -45,6 +44,8 @@ const static char _serial_c_Id [] = "$Id: serial.c,v 1.24.2.1 2003/06/02 11:24:3
 #include <clplumbing/longclock.h>
 #include <clplumbing/timers.h>
 
+
+
 #define PIL_PLUGINTYPE		HB_COMM_TYPE
 #define PIL_PLUGINTYPE_S	HB_COMM_TYPE_S
 #define PIL_PLUGIN		serial
@@ -68,10 +69,10 @@ static const char *	baudstring;
 static struct hb_media*		lastserialport;
 
 static struct hb_media*	serial_new(const char * value);
-static struct ha_msg*	serial_read(struct hb_media *mp);
+static void*		serial_read(struct hb_media *mp, int* lenp);
 static char *		ttygets(char * inbuf, int length
 ,				struct serial_private *tty);
-static int		serial_write(struct hb_media*mp, struct ha_msg *msg);
+static int		serial_write(struct hb_media*mp, void *msg , int len);
 static int		serial_open(struct hb_media* mp);
 static int		ttysetup(int fd, const char * ourtty);
 static int		opentty(char * serial_device);
@@ -132,6 +133,7 @@ static void*			interfprivate;
 
 #define LOG	PluginImports->log
 #define MALLOC	PluginImports->alloc
+#define STRDUP  PluginImports->mstrdup
 #define FREE	PluginImports->mfree
 
 PIL_rc
@@ -163,30 +165,30 @@ PIL_PLUGIN_INIT(PILPlugin*us, const PILPluginImports* imports)
 }
 
 #define		IsTTYOBJECT(mp)	((mp) && ((mp)->vf == (void*)&serial_media_fns))
-//#define		TTYASSERT(mp)	ASSERT(IsTTYOBJECT(mp))
+/*
+#define		TTYASSERT(mp)	ASSERT(IsTTYOBJECT(mp))
+*/
 #define		TTYASSERT(mp)
 #define		RTS_WARNTIME	3600
 
 static int
 serial_mtype (char **buffer) { 
-	
-	*buffer = MALLOC((strlen("serial") * sizeof(char)) + 1);
+	*buffer = STRDUP("serial");
+	if (!*buffer) {
+		return 0;
+	}
 
-	strcpy(*buffer, "serial");
-
-	return strlen("serial");
+	return strlen(*buffer);
 }
 
 static int
 serial_descr (char **buffer) { 
+	*buffer = STRDUP("serial ring");
+	if (!*buffer) {
+		return 0;
+	}
 
-	const char *str = "serial ring";	
-
-	*buffer = MALLOC((strlen(str) * sizeof(char)) + 1);
-
-	strcpy(*buffer, str);
-
-	return strlen(str);
+	return strlen(*buffer);
 }
 
 static int
@@ -198,9 +200,6 @@ serial_isping (void) {
 static int
 serial_init (void)
 {
-	(void)_serial_c_Id;
-	(void)_heartbeat_h_Id;
-	(void)_ha_msg_h_Id;	/* ditto */
 	lastserialport = NULL;
 
 	/* This eventually ought be done through the configuration API */
@@ -214,7 +213,7 @@ serial_init (void)
 		baudstring  = DEFAULTBAUDSTR;
 	}
 	if (ANYDEBUG) {
-		LOG(PIL_DEBUG, "serial_init: serial_baud = 0x%x"
+		PILCallLog(LOG, PIL_DEBUG, "serial_init: serial_baud = 0x%x"
 		,	serial_baud);
 	}
 	return(HA_OK);
@@ -230,19 +229,19 @@ serial_new (const char * port)
 
 	/* Let's see if this looks like it might be a serial port... */
 	if (*port != '/') {
-		LOG(PIL_CRIT
+		PILCallLog(LOG, PIL_CRIT
 		,	"Serial port not full pathname [%s] in config file"
 		,	port);
 		return(NULL);
 	}
 
 	if (stat(port, &sbuf) < 0) {
-		LOG(PIL_CRIT, "Nonexistent serial port [%s] in config file"
+		PILCallLog(LOG, PIL_CRIT, "Nonexistent serial port [%s] in config file"
 		,	port);
 		return(NULL);
 	}
 	if (!S_ISCHR(sbuf.st_mode)) {
-		LOG(PIL_CRIT
+		PILCallLog(LOG, PIL_CRIT
 		,	"Serial port [%s] not a char device in config file"
 		,	port);
 		return(NULL);
@@ -261,18 +260,23 @@ serial_new (const char * port)
 			 */
 			sp->next = lastserialport;
 			lastserialport=ret;
-			sp->ttyname = (char *)MALLOC(strlen(port)+1);
-			sp->consecutive_errors = 0;
-			strcpy(sp->ttyname, port);
-			ret->name = sp->ttyname;
-			ret->pd = sp;
-		}else{
+			sp->ttyname = STRDUP(port);
+			if (sp->ttyname != NULL) {
+				sp->consecutive_errors = 0;
+				ret->name = sp->ttyname;
+				ret->pd = sp;
+			}else{
+				FREE(sp);
+				sp = NULL;
+			}
+		}
+		if (sp == NULL) {
 			FREE(ret);
 			ret = NULL;
-			LOG(PIL_CRIT, "Out of memory (private serial data)");
+			PILCallLog(LOG, PIL_CRIT, "Out of memory (private serial data)");
 		}
 	}else{
-		LOG(PIL_CRIT, "Out of memory (serial data)");
+		PILCallLog(LOG, PIL_CRIT, "Out of memory (serial data)");
 	}
 	return(ret);
 }
@@ -285,13 +289,13 @@ serial_open (struct hb_media* mp)
 	TTYASSERT(mp);
 	sp = (struct serial_private*)mp->pd;
 	if (OurImports->devlock(sp->ttyname) < 0) {
-		LOG(PIL_CRIT, "cannot lock line %s", sp->ttyname);
+		PILCallLog(LOG, PIL_CRIT, "cannot lock line %s", sp->ttyname);
 		return(HA_FAIL);
 	}
 	if ((sp->ttyfd = opentty(sp->ttyname)) < 0) {
 		return(HA_FAIL);
 	}
-	LOG(PIL_INFO, "Starting serial heartbeat on tty %s (%s baud)"
+	PILCallLog(LOG, PIL_INFO, "Starting serial heartbeat on tty %s (%s baud)"
 	,	sp->ttyname, baudstring);
 	return(HA_OK);
 }
@@ -316,7 +320,7 @@ ttysetup(int fd, const char * ourtty)
 	struct TERMIOS	ti;
 
 	if (GETATTR(fd, &ti) < 0) {
-		LOG(PIL_CRIT, "cannot get tty attributes: %s", strerror(errno));
+		PILCallLog(LOG, PIL_CRIT, "cannot get tty attributes: %s", strerror(errno));
 		return(HA_FAIL);
 	}
 
@@ -358,17 +362,17 @@ ttysetup(int fd, const char * ourtty)
 	ti.c_cc[VMIN] = 1;
 	ti.c_cc[VTIME] = 1;
 	if (SETATTR(fd, &ti) < 0) {
-		LOG(PIL_CRIT, "cannot set tty attributes: %s"
+		PILCallLog(LOG, PIL_CRIT, "cannot set tty attributes: %s"
 		,	strerror(errno));
 		return(HA_FAIL);
 	}
 	if (ANYDEBUG) {
-		LOG(PIL_DEBUG, "tty setup on %s complete.", ourtty);
-		LOG(PIL_DEBUG, "Baud rate set to: 0x%x", serial_baud);
-		LOG(PIL_DEBUG, "ti.c_iflag = 0x%x", ti.c_iflag);
-		LOG(PIL_DEBUG, "ti.c_oflag = 0x%x", ti.c_oflag);
-		LOG(PIL_DEBUG, "ti.c_cflag = 0x%x", ti.c_cflag);
-		LOG(PIL_DEBUG, "ti.c_lflag = 0x%x", ti.c_lflag);
+		PILCallLog(LOG, PIL_DEBUG, "tty setup on %s complete.", ourtty);
+		PILCallLog(LOG, PIL_DEBUG, "Baud rate set to: 0x%x", serial_baud);
+		PILCallLog(LOG, PIL_DEBUG, "ti.c_iflag = 0x%x", ti.c_iflag);
+		PILCallLog(LOG, PIL_DEBUG, "ti.c_oflag = 0x%x", ti.c_oflag);
+		PILCallLog(LOG, PIL_DEBUG, "ti.c_cflag = 0x%x", ti.c_cflag);
+		PILCallLog(LOG, PIL_DEBUG, "ti.c_lflag = 0x%x", ti.c_lflag);
 	}
 	/* For good measure */
 	FLUSH(fd);
@@ -387,7 +391,7 @@ opentty(char * serial_device)
 	int	fd;
 
 	if ((fd=open(serial_device, O_RDWR|O_NOCTTY)) < 0 ) {
-		LOG(LOG_CRIT, "cannot open %s: %s", serial_device
+		PILCallLog(LOG, PIL_CRIT, "cannot open %s: %s", serial_device
 		,	strerror(errno));
 		return(fd);
 	}
@@ -396,7 +400,7 @@ opentty(char * serial_device)
 		return(-1);
 	}
 	if (fcntl(fd, F_SETFD, FD_CLOEXEC)) {
-		LOG(PIL_WARN,"Error setting the close-on-exec flag: %s"
+		PILCallLog(LOG, PIL_WARN,"Error setting the close-on-exec flag: %s"
 		,	strerror(errno));
 	}
 	/* Cause the other guy to flush his I/O */
@@ -416,26 +420,178 @@ serial_localdie(void)
 	ourtty = ((struct serial_private*)(ourmedia->pd))->ttyfd;
 	if (ourtty >= 0) {
 		if (ANYDEBUG) {
-			LOG(PIL_DEBUG, "serial_localdie: Flushing tty");
+			PILCallLog(LOG, PIL_DEBUG, "serial_localdie: Flushing tty");
 		}
 		tcflush(ourtty, TCIOFLUSH);
 	}
 }
 
-/* This function does all the writing to our tty ports */
-static int
-serial_write (struct hb_media*mp, struct ha_msg*m)
-{
-	char *			str;
 
+/* This function does all the reading from our tty ports */
+
+static void *
+serial_read(struct hb_media* mp, int *lenp)
+{
+	char			buf[MAXLINE];
+	struct serial_private*	thissp;
+	int			startlen;
+	const char *		start = MSG_START;
+	const char *		end = MSG_END;
+	int			endlen;
+	char			*msgstring;
+	char			*p;
+	int			len = 0;
+	int			tmplen;
+
+
+	
+	
+	TTYASSERT(mp);
+	thissp = (struct serial_private*)mp->pd;
+
+	startlen = strlen(start);
+	if (start[startlen-1] == '\n') {
+		--startlen;
+	}
+	endlen = strlen(end);
+	if (end[endlen-1] == '\n') {
+		--endlen;
+	}
+	
+	msgstring = ha_calloc(MAXMSG,1 );
+	if(!msgstring){
+		PILCallLog(LOG, PIL_CRIT, "serial_read: cannot allocate memory to msgstring ");
+		return(NULL);
+	}
+	msgstring[0] = 0;
+
+	p = msgstring;
+	
+	/* Skip until we find a MSG_START (hopefully we skip nothing) */
+	while (ttygets(buf, MAXLINE, thissp) != NULL
+	       &&	strncmp(buf, start, startlen) != 0) {
+		
+		
+		/*nothing*/
+	}
+	
+	len = strnlen(buf, MAXLINE) + 1;
+	if(len >=  MAXMSG){
+		PILCallLog(LOG, PIL_CRIT,  "serial_read:MSG_START exceeds MAXMSG");
+		ha_free(msgstring);
+		return(NULL);
+	}
+
+	tmplen = strnlen(buf, MAXLINE);
+	
+	strcat(p, buf);
+	p += tmplen;
+	strcat(p, "\n");
+	p++;
+
+	while (ttygets(buf, MAXLINE, thissp) != NULL
+	       &&	strncmp(buf, MSG_END, endlen) != 0) {
+		
+		
+		len += strnlen(buf, MAXLINE) + 1;
+		if(len >= MAXMSG){
+			PILCallLog(LOG, PIL_CRIT, "serial_read:msgstring exceeds MAXMSG");
+			ha_free(msgstring);
+			return(NULL);
+		}
+		
+		
+		tmplen = strnlen(buf, MAXLINE);
+		memcpy(p, buf, tmplen);		
+		p += tmplen;
+		strcat(p, "\n");
+		p++;
+	}
+
+	
+	if(strncmp(buf, MSG_END, endlen) == 0){
+		
+		len += strnlen(buf, MAXLINE) + 2;
+		if(len >= MAXMSG){
+			PILCallLog(LOG, PIL_CRIT, "serial_read:msgstring exceeds MAXMSG after adding MSG_END");
+			ha_free(msgstring);
+			return(NULL);
+		}
+
+		tmplen = strnlen(buf, MAXLINE);
+		
+		memcpy(p, buf, tmplen);
+		p += tmplen;
+		strcat(p, "\n");
+		p++;
+		p[0] = 0;
+	}
+	
+	if (buf[0] == EOS ) {
+		ha_free(msgstring);
+		return NULL;
+	}else{
+		thissp->consecutive_errors=0;
+	}
+	
+	
+	*lenp = len;
+	
+	/*
+	PILCallLog(LOG, PIL_INFO, "final msgstring=%s", msgstring);
+	*/
+	
+	return(msgstring);	
+}
+
+/* This function does all the writing to our tty ports */
+
+static int
+serial_write(struct hb_media* mp, void *p, int len)
+{
+
+	int			string_startlen = sizeof(MSG_START)-1;
+	int			netstring_startlen = sizeof(MSG_START_NETSTRING) - 1;
+
+	char			*str;
+	int			str_new = 0;
 	int			wrc;
 	int			size;
 	int			ourtty;
 	static gboolean		warnyet=FALSE;
 	static longclock_t	warninterval;
 	static longclock_t	lastwarn;
+	
+
+	if (strncmp(p, MSG_START, string_startlen) == 0) {
+		str = p;
+		size = strlen(str);
+		if(size > len){
+			return(HA_FAIL);
+		}
+	} else if(strncmp(p, MSG_START_NETSTRING, netstring_startlen) == 0) {
+		
+		struct ha_msg * msg;
+		
+		msg = wirefmt2msg(p, len);
+		if(!msg){
+			ha_log(PIL_WARN, "serial_write(): wirefmt2msg() failed");
+			return(HA_FAIL);
+		}
+		
+		add_msg_auth(msg);
+		str = msg2string(msg);		
+		str_new = 1;
+		size = strlen(str);
+		ha_msg_del(msg);
+		
+	} else{
+		return(HA_FAIL);
+	}
+
 
 	TTYASSERT(mp);
+	
 
 	if (!warnyet) {
 		warninterval = msto_longclock(RTS_WARNTIME*1000L);
@@ -443,28 +599,24 @@ serial_write (struct hb_media*mp, struct ha_msg*m)
 	ourmedia = mp;	/* Only used for the "localdie" function */
 	OurImports->RegisterCleanup(serial_localdie);
 	ourtty = ((struct serial_private*)(mp->pd))->ttyfd;
-	if ((str=msg2string(m)) == NULL) {
-		LOG(PIL_CRIT, "Cannot convert message to tty string");
-		return(HA_FAIL);
-	}
-	size = strlen(str);
+
 	if (DEBUGPKT) {
-		LOG(PIL_DEBUG, "Sending pkt to %s [%d bytes]"
+		PILCallLog(LOG, PIL_DEBUG, "Sending pkt to %s [%d bytes]"
 		,	mp->name, size);
 	}
 	if (DEBUGPKTCONT) {
-		LOG(PIL_DEBUG, str);
+		PILCallLog(LOG, PIL_DEBUG, "%s", str);
 	}
 	setmsalarm(500);
 	wrc = write(ourtty, str, size);
 	cancelmstimer();
 	if (DEBUGPKTCONT) {
-		LOG(PIL_DEBUG, "serial write returned %d", wrc);
+		PILCallLog(LOG, PIL_DEBUG, "serial write returned %d", wrc);
 	}
-
+	
 	if (wrc < 0 || wrc != size) {
 		if (DEBUGPKTCONT && wrc < 0) {
-			LOG(PIL_DEBUG, "serial write errno was %d", errno);
+			PILCallLog(LOG, PIL_DEBUG, "serial write errno was %d", errno);
 		}
 		if (wrc > 0 || (wrc < 0 && errno == EINTR)) {
 			longclock_t	now = time_longclock();
@@ -476,152 +628,25 @@ serial_write (struct hb_media*mp, struct ha_msg*m)
 
 				lastwarn = now;
 				warnyet = TRUE;
-				LOG(LOG_ERR
+				PILCallLog(LOG, PIL_WARN
 				,	"TTY write timeout on [%s]"
 				" (no connection or bad cable"
 				"? [see documentation])"
 				,	mp->name);
 			}
 		}else{
-			LOG(PIL_CRIT, "TTY write failure on [%s]: %s"
-			,	mp->name, strerror(errno));
+			PILCallLog(LOG, PIL_CRIT, "TTY write failure on [%s]: %s"
+			    ,	mp->name, strerror(errno));
 		}
 	}
-	ha_free(str);
+	
+	if(str_new){
+		ha_free(str);
+		str = NULL;
+	}
 	return(HA_OK);
 }
 
-/* This function does all the reading from our tty ports */
-static struct ha_msg *
-serial_read (struct hb_media*mp)
-{
-	char buf[MAXLINE];
-	const char *		bufmax = buf + sizeof(buf);
-	struct hb_media*	sp;
-	struct serial_private*	thissp;
-	struct ha_msg*		ret;
-	char *			newmsg = NULL;
-	int			newmsglen = 0;
-	int			startlen;
-	const char *		start = MSG_START;
-	const char *		end = MSG_START;
-	int			endlen;
-	struct serial_private*	spp;
-
-	TTYASSERT(mp);
-	thissp = (struct serial_private*)mp->pd;
-
-	if ((ret = ha_msg_new(0)) == NULL) {
-		LOG(PIL_CRIT, "Cannot get new message");
-		return(NULL);
-	}
-	startlen = strlen(start);
-	if (start[startlen-1] == '\n') {
-		--startlen;
-	}
-	endlen = strlen(end);
-	if (end[endlen-1] == '\n') {
-		--endlen;
-	}
-	/* Skip until we find a MSG_START (hopefully we skip nothing) */
-	while (ttygets(buf, MAXLINE, thissp) != NULL
-	&&	strncmp(buf, start, startlen) != 0) {
-		/* Nothing */
-	}
-
-	while (ttygets(buf, MAXLINE, thissp) != NULL
-	&&	strncmp(buf, MSG_END, endlen) != 0) {
-
-		/* Add the "name=value" string on this line to the message */
-		if (ha_msg_add_nv(ret, buf, bufmax) != HA_OK) {
-			ha_msg_del(ret);
-			return(NULL);
-		}
-	}
-
-	if (buf[0] == EOS || ret->nfields < 1) {
-		ha_msg_del(ret);
-		return NULL;
-	}else{
-		thissp->consecutive_errors=0;
-	}
-
-	/* Should this message should continue around the ring? */
-
-	if (!isauthentic(ret) || !should_ring_copy_msg(ret)) {
-		/* Avoid infinite loops... Ignore this message */
-		return(ret);
-	}
-
-
-	/* Add Name=value pairs until we reach MSG_END or EOF */
-	/* Forward message to other port in ring (if any) */
-	for (sp=lastserialport; sp; sp=spp->next) {
-		TTYASSERT(sp);
-		spp = (struct serial_private*)sp->pd;
-		if (sp == mp) {
-			/* That's us! */
-			continue;
-		}
-
-		/* Modify message, decrementing TTL (and reauthenticate it) */
-		if (newmsglen) {
-			const char *		ttl_s;
-			int			ttl;
-			char			nttl[8];
-
-			/* Decrement TTL in the message before forwarding */
-			if ((ttl_s = ha_msg_value(ret, F_TTL)) == NULL) {
-				return(ret);
-			}
-			ttl = atoi(ttl_s);
-			sprintf(nttl, "%d", ttl-1);
-			ha_msg_mod(ret, F_TTL, nttl);
-
-			/* Re-authenticate message */
-			add_msg_auth(ret);
-
-			if ((newmsg = msg2string(ret)) == NULL) {
-				LOG(PIL_CRIT, "Cannot convert serial msg to string");
-				continue;
-			}
-			newmsglen = strlen(newmsg);
-		}
-		/*
-		 * This will eventually have to be changed
-		 * if/when we change from FIFOs to more general IPC
-		 */
-		if (newmsglen) {
-			/*
-			 * I suppose it just becomes an IPC abstraction
-			 * and we issue a "msgput" or some such on it...
-			 */
-#if 0
-			if (DEBUGPKT) {
-				ha_log(LOG_DEBUG
-				,	"serial_read: writing %s"
-				" (len: %d) to %d"
-				,	newmsg, newmsglen
-				,	sp->wpipe[P_WRITEFD]);
-			}
-			/* FIXME!!!!! */
-			/* I think the code as it stands never calls this
-			 * anyway.  It *should* call this, but I don't
-			 * see any way for newmsglen to become nonzero
-			 */
-			write(sp->wchan[P_WRITEFD], newmsg, newmsglen);
-#else
-			;
-#endif
-		}
-	}
-
-	if (newmsglen) {
-		ha_free(newmsg);
-	}
-	return(ret);
-}
-
 
 /* Gets function for our tty */
 static char *
@@ -642,16 +667,16 @@ ttygets(char * inbuf, int length, struct serial_private *tty)
 		errno = saverr;
 		if (rc != 1) {
 			if (rc == 0 || errno == EINTR) {
-				LOG(PIL_CRIT, "EOF in ttygets [%s]: %s [%d]"
+				PILCallLog(LOG, PIL_WARN, "EOF in ttygets [%s]: %s [%d]"
 				,	tty->ttyname, strerror(errno), rc);
 				++tty->consecutive_errors;
 				tcsetpgrp(fd, getsid(getpid()));
 				if ((tty->consecutive_errors % 10) == 0) {
-					LOG(PIL_WARN
+					PILCallLog(LOG, PIL_WARN
 					,	"10 consecutive EOF"
 					" errors from serial port %s"
 					,	tty->ttyname);
-					LOG(PIL_INFO
+					PILCallLog(LOG, PIL_INFO
 					,	"%s pgrp: %d", tty->ttyname
 					,	tcgetpgrp(fd));
 					sleep(10);
@@ -673,11 +698,59 @@ ttygets(char * inbuf, int length, struct serial_private *tty)
 }
 /*
  * $Log: serial.c,v $
- * Revision 1.24.2.1  2003/06/02 11:24:37  lars
- * Merge from CVS head:
- * - Put in a bug fix from Carson Gaspar <carson@taltos.org> to open our
- *   serial ttys with O_NOCTTY to avoid having them become our controlling
- *   ttys.
+ * Revision 1.29.2.4  2004/05/12 02:04:38  alan
+ * Changed a message back from an error into a warning.
+ *
+ * Revision 1.29.2.3  2004/05/12 01:46:26  alan
+ * Changed tty write timeout back to warning.
+ *
+ * Revision 1.29.2.2  2004/05/12 00:10:14  alan
+ * Changed all the HBcomm plugins to use PILCallLog() for logging
+ * instead of calling the function pointer directly.
+ * Also, in the process fixed several mismatches between arguments
+ * and format strings, and a couple of format string vulnerabilities.
+ *
+ * Revision 1.34  2004/05/11 22:04:35  alan
+ * Changed all the HBcomm plugins to use PILCallLog() for logging instead of calling
+ * the function pointer directly.
+ * Also, in the process fixed several mismatches between arguments and format strings, and
+ * a couple of format string vulnerabilities.
+ *
+ * Revision 1.33  2004/04/29 16:24:34  msoffen
+ * fixed comment in Log
+ *
+ * Revision 1.32  2004/04/29 16:23:16  msoffen
+ * Changed // comments to / * * / comments.
+ *
+ * Revision 1.31  2004/03/03 05:31:51  alan
+ * Put in Gochun Shi's new netstrings on-the-wire data format code.
+ * this allows sending binary data, among many other things!
+ *
+ * Revision 1.30  2004/02/17 22:11:59  lars
+ * Pet peeve removal: _Id et al now gone, replaced with consistent Id header.
+ *
+ * Revision 1.29  2004/01/21 11:34:15  horms
+ * - Replaced numerous malloc + strcpy/strncpy invocations with strdup
+ *   * This usually makes the code a bit cleaner
+ *   * Also is easier not to make code with potential buffer over-runs
+ * - Added STRDUP to pils modules
+ * - Removed some spurious MALLOC and FREE redefinitions
+ *   _that could never be used_
+ * - Make sure the return value of strdup is honoured in error conditions
+ *
+ * Revision 1.28  2004/01/20 16:23:11  alan
+ * Fixed an oversight which turns out to have been luckily-benign in practice.
+ *
+ * Revision 1.27  2003/05/21 21:56:50  alan
+ * Minor tweak to the last tty fix from Carson Gaspar.
+ *
+ * Revision 1.26  2003/05/21 21:55:07  alan
+ * Put in a bug fix from Carson Gaspar <carson@taltos.org> to open our
+ * serial ttys with O_NOCTTY to avoid having them become our controlling
+ * ttys.
+ *
+ * Revision 1.25  2003/04/15 23:09:52  alan
+ * Continuing saga of the semi-major heartbeat process restructure.
  *
  * Revision 1.24  2003/02/07 08:37:18  horms
  * Removed inclusion of portability.h from .h files
