@@ -1,4 +1,4 @@
-/* $Id: ucast.c,v 1.17.2.3 2004/05/12 00:10:14 alan Exp $ */
+/* $Id: ucast.c,v 1.17.2.4 2005/01/12 02:25:47 horms Exp $ */
 /*
  * Adapted from alanr's UDP broadcast heartbeat bcast.c by Stphane Billiart
  *	<stephane@reefedge.com>
@@ -388,7 +388,7 @@ static void *
 ucast_read(struct hb_media* mp, int *lenp)
 {
 	struct ip_private *ei;
-	int addr_len;
+	socklen_t addr_len;
 	struct sockaddr_in their_addr;
 	int numbytes;
 	char buf[MAXLINE];
