@@ -1,4 +1,4 @@
-/* $Id: apcmastersnmp.c,v 1.8.2.3 2004/09/11 06:36:33 msoffen Exp $ */
+/* $Id: apcmastersnmp.c,v 1.8.2.5 2005/04/27 18:58:41 blaschke Exp $ */
 /*
  * Stonith module for APC Masterswitch (SNMP)
  * Copyright (c) 2001 Andreas Piesk <a.piesk@gmx.net>
@@ -103,7 +103,7 @@ static struct stonith_ops apcmastersnmpOps ={
 	apcmastersnmp_set_config_info,	/* Get configuration from file	*/
 	apcmastersnmp_getinfo,		/* Return STONITH info string	*/
 	apcmastersnmp_status,		/* Return STONITH device status	*/
-	apcmastersnmp_reset_req,		/* Request a reset */
+	apcmastersnmp_reset_req,	/* Request a reset */
 	apcmastersnmp_hostlist,		/* Return list of supported hosts */
 	apcmastersnmp_free_hostlist	/* free above list */
 };
@@ -210,12 +210,30 @@ static const char *NOTapcID = "destroyed (APCMasterswitch)";
  * own prototypes 
  */
 
+void APC_error(struct snmp_session *sptr, const char *fn, const char *msg);
 struct snmp_session *APC_open(char *hostname, int port, char *community);
 int APC_parse_config_info(struct APCDevice *ad, const char *info);
 void *APC_read(struct snmp_session *sptr, const char *objname, int type);
 int APC_write(struct snmp_session *sptr, const char *objname, char type,
 	      char *value);
 
+
+/*
+ *  log snmp error
+ */
+void APC_error(struct snmp_session *sptr, const char *fn, const char *msg)
+{
+    int snmperr = 0;
+    int cliberr = 0;
+    char *errstr;
+
+    snmp_error(sptr, &cliberr, &snmperr, &errstr);
+    syslog(LOG_ERR,
+        "%s: %s (cliberr: %i / snmperr: %i / error: %s).",
+	fn, msg, cliberr, snmperr, errstr);
+    free(errstr);
+}
+
 /*
  *  creates a snmp session
  */
@@ -224,12 +242,6 @@ struct snmp_session *APC_open(char *hostname, int port, char *community)
     static struct snmp_session session;
     struct snmp_session *sptr;
 
-#ifdef APC_DEBUG
-    int snmperr = 0;
-    int cliberr = 0;
-    char *errstr;
-#endif
-
     /* create session */
     snmp_sess_init(&session);
 
@@ -237,7 +249,7 @@ struct snmp_session *APC_open(char *hostname, int port, char *community)
     session.peername = hostname;
     session.version = SNMP_VERSION_1;
     session.remote_port = port;
-    session.community = community;
+    session.community = (unsigned char*) community;
     session.community_len = strlen(community);
     session.retries = 5;
     session.timeout = 1000000;
@@ -245,15 +257,9 @@ struct snmp_session *APC_open(char *hostname, int port, char *community)
     /* open session */
     sptr = snmp_open(&session);
 
-#ifdef APC_DEBUG
     if (sptr == NULL) {
-	snmp_error(&session, &cliberr, &snmperr, &errstr);
-	syslog(LOG_DEBUG,
-	       "%s: open error (cliberr: %i / snmperr: %i / error: %s\n",
-	       __FUNCTION__, cliberr, snmperr, errstr);
-	free(errstr);
+        APC_error(&session, __FUNCTION__, "cannot open snmp session");
     }
-#endif
 
     /* return pointer to opened session */
     return (sptr);
@@ -284,8 +290,8 @@ APC_parse_config_info(struct APCDevice *ad, const char *info)
 	/* try to resolve the hostname/ip-address */
 	if (gethostbyname(hostname) != NULL) {
 
-        /* init snmp library */
-        init_snmp("apcmastersnmp");
+            /* init snmp library */
+            init_snmp("apcmastersnmp");
 
 	    /* now try to get a snmp session */
 	    if ((ad->sptr = APC_open(hostname, port, community)) != NULL) {
@@ -293,32 +299,30 @@ APC_parse_config_info(struct APCDevice *ad, const char *info)
 		/* ok, get the number of outlets from the masterswitch */
 		if ((i = APC_read(ad->sptr, OID_NUM_OUTLETS, ASN_INTEGER))
 		    == NULL) {
-#ifdef APC_DEBUG
-		    syslog(LOG_DEBUG, "%s: cannot read number of outlets.",
+		    syslog(LOG_ERR, "%s: cannot read number of outlets.",
 			   __FUNCTION__);
-#endif
 		    return (S_ACCESS);
 		}
 		/* store the number of outlets */
 		ad->num_outlets = *i;
 #ifdef APC_DEBUG
-		syslog(LOG_DEBUG, "%s: number of outlets: %i",
+		syslog(LOG_DEBUG, "%s: number of outlets: %i.",
 			   __FUNCTION__, ad->num_outlets );
 #endif
 
-
-		/* everythin went well */
+		/* everything went well */
 		return (S_OK);
-	    }
-#ifdef APC_DEBUG
-	    syslog(LOG_DEBUG, "%s: cannot create snmp session",
-		   __FUNCTION__);
-#endif
-	}
-#ifdef APC_DEBUG
-	syslog(LOG_DEBUG, "%s: cannot resolve hostname '%s'", __FUNCTION__,
-	       hostname);
-#endif
+	    }else{
+	        syslog(LOG_ERR, "%s: cannot create snmp session.",
+	            __FUNCTION__);
+            }
+	}else{
+            syslog(LOG_ERR, "%s: cannot resolve hostname '%s', h_errno %d.",
+               __FUNCTION__, hostname, h_errno);
+        }
+    }else{
+        syslog(LOG_ERR, "%s: cannot parse config info '%s'.", __FUNCTION__,
+            info);
     }
     /* no valid config */
     return (S_BADCONFIG);
@@ -338,12 +342,14 @@ void *APC_read(struct snmp_session *sptr, const char *objname, int type)
     static int response_int;
 
 #ifdef APC_DEBUG
-    syslog(LOG_DEBUG, "%s: requested objname '%s'", __FUNCTION__, objname );
+    syslog(LOG_DEBUG, "%s: requested objname '%s'.", __FUNCTION__, objname );
 #endif
 
     /* convert objname into oid; return NULL if invalid */
-    if (!read_objid(objname, name, &namelen))
+    if (!read_objid(objname, name, &namelen)) {
+        syslog(LOG_ERR, "%s: cannot convert %s to oid.", __FUNCTION__, objname);
 	return (NULL);
+    }
 
     /* create pdu */
     if ((pdu = snmp_pdu_create(SNMP_MSG_GET)) != NULL) {
@@ -364,7 +370,7 @@ void *APC_read(struct snmp_session *sptr, const char *objname, int type)
 		    /* return response as string */
 		    if ((vars->type == type) && (type == ASN_OCTET_STR)) {
 			memset(response_str, 0, MAX_STRING);
-			strncpy(response_str, vars->val.string,
+			strncpy(response_str, (char*) vars->val.string,
 				MIN(vars->val_len, MAX_STRING));
 			snmp_free_pdu(resp);
 			return ((void *) response_str);
@@ -376,16 +382,17 @@ void *APC_read(struct snmp_session *sptr, const char *objname, int type)
 			return ((void *) &response_int);
 		    }
 		}
-
-#ifdef APC_DEBUG
-	    } else {
-		syslog(LOG_DEBUG, "%s: Error in packet - Reason: %s", __FUNCTION__,
-		       snmp_errstring(resp->errstat));
-#endif
+	    }else{
+		syslog(LOG_ERR, "%s: error in response packet, reason %ld [%s].",
+                    __FUNCTION__, resp->errstat, snmp_errstring(resp->errstat));
 	    }
-	}
-	/* free repsonse pdu (neccessary?) */
+	}else{
+            APC_error(sptr, __FUNCTION__, "error sending/receiving pdu");
+        }
+	/* free repsonse pdu (necessary?) */
 	snmp_free_pdu(resp);
+    }else{
+        APC_error(sptr, __FUNCTION__, "cannot create pdu");
     }
     /* error: return nothing */
     return (NULL);
@@ -404,12 +411,14 @@ APC_write(struct snmp_session *sptr, const char *objname, char type,
     struct snmp_pdu *resp;
 
 #ifdef APC_DEBUG
-    syslog(LOG_DEBUG, "%s: requested objname '%s'", __FUNCTION__, objname );
+    syslog(LOG_DEBUG, "%s: requested objname '%s'.", __FUNCTION__, objname );
 #endif
 
     /* convert objname into oid; return NULL if invalid */
-    if (!read_objid(objname, name, &namelen))
+    if (!read_objid(objname, name, &namelen)) {
+        syslog(LOG_ERR, "%s: cannot convert %s to oid.", __FUNCTION__, objname);
 	return (FALSE);
+    }
 
     /* create pdu */
     if ((pdu = snmp_pdu_create(SNMP_MSG_SET)) != NULL) {
@@ -427,15 +436,17 @@ APC_write(struct snmp_session *sptr, const char *objname, char type,
 		snmp_free_pdu(resp);
 		return (TRUE);
 
-#ifdef APC_DEBUG
 	    } else {
-		syslog(LOG_DEBUG, "%s: Error in packet- Reason: %s", __FUNCTION__,
-		       snmp_errstring(resp->errstat));
-#endif
+		syslog(LOG_ERR, "%s: error in response packet, reason %ld [%s].",
+                    __FUNCTION__, resp->errstat, snmp_errstring(resp->errstat));
 	    }
+	}else{
+            APC_error(sptr, __FUNCTION__, "error sending/receiving pdu");
 	}
-	/* free pdu (again: neccessary?) */
+	/* free pdu (again: necessary?) */
 	snmp_free_pdu(resp);
+    }else{
+        APC_error(sptr, __FUNCTION__, "cannot create pdu");
     }
     /* error */
     return (FALSE);
@@ -468,9 +479,7 @@ int apcmastersnmp_status(Stonith * s)
     ad = (struct APCDevice *) s->pinfo;
 
     if ((ident = APC_read(ad->sptr, OID_IDENT, ASN_OCTET_STR)) == NULL) {
-#ifdef APC_DEBUG
-	syslog(LOG_DEBUG, "%s: cannot read ident.", __FUNCTION__);
-#endif
+	syslog(LOG_ERR, "%s: cannot read ident.", __FUNCTION__);
 	return (S_ACCESS);
     }
 
@@ -479,7 +488,7 @@ int apcmastersnmp_status(Stonith * s)
       if (strcmp(ident, APC_tested_ident[i]) == 0) break;
     if (i<0) {
 	syslog(LOG_WARNING,
-	       "%s: module not tested with this hardware '%s'",
+	       "%s: module not tested with this hardware '%s'.",
 	       __FUNCTION__, ident);
     }
     /* status ok */
@@ -533,6 +542,8 @@ apcmastersnmp_hostlist(Stonith * s)
 	/* read outlet name */
 	if ((outlet_name = APC_read(ad->sptr, objname, ASN_OCTET_STR)) ==
 	    NULL) {
+            syslog(LOG_ERR, "%s: cannot read name for outlet %d.",
+                __FUNCTION__, j+1);
 	    apcmastersnmp_free_hostlist(hl);
 	    hl = NULL;
 	    return (hl);
@@ -547,7 +558,7 @@ apcmastersnmp_hostlist(Stonith * s)
 	if (h >= num_outlets) {
 		/* put outletname in hostlist */
 #ifdef APC_DEBUG
-	        syslog(LOG_DEBUG, "%s: added %s to hostlist", __FUNCTION__,
+	        syslog(LOG_DEBUG, "%s: added %s to hostlist.", __FUNCTION__,
 				outlet_name);
 #endif
 		
@@ -563,7 +574,7 @@ apcmastersnmp_hostlist(Stonith * s)
 
 
 #ifdef APC_DEBUG
-    syslog(LOG_DEBUG, "%s: %d unique hosts connected to %d outlets", 
+    syslog(LOG_DEBUG, "%s: %d unique hosts connected to %d outlets.", 
 		    __FUNCTION__, num_outlets, j);
 #endif
     /* return list */
@@ -643,10 +654,8 @@ apcmastersnmp_reset_req(Stonith * s, int request, const char *host)
 	/* read outlet name */
 	if ((outlet_name = APC_read(ad->sptr, objname, ASN_OCTET_STR)) ==
 	    NULL) {
-#ifdef APC_DEBUG
-	    syslog(LOG_DEBUG, "%s: cannot read outlet_names.",
-		   __FUNCTION__);
-#endif
+	    syslog(LOG_ERR, "%s: cannot read name for outlet %d.",
+		   __FUNCTION__, outlet);
 	    return (S_ACCESS);
 	}
 	
@@ -654,7 +663,7 @@ apcmastersnmp_reset_req(Stonith * s, int request, const char *host)
 	g_strdown(outlet_name);
 	if (strcmp(outlet_name, host) == 0) {
 #ifdef APC_DEBUG
-	    	syslog(LOG_DEBUG, "%s: Found %s at outlet: %i",
+	    	syslog(LOG_DEBUG, "%s: found %s at outlet %i.",
 		       __FUNCTION__, host, outlet);
 #endif
 		/* Check that the outlet is not administratively down */
@@ -664,9 +673,8 @@ apcmastersnmp_reset_req(Stonith * s, int request, const char *host)
 
 		/* get outlet's state */
 		if ((state = APC_read(ad->sptr, objname, ASN_INTEGER)) == NULL) {
-#ifdef APC_DEBUG
-			syslog(LOG_DEBUG, "%s: cannot read outlet_state for outlet %d.", __FUNCTION__, outlet);
-#endif
+			syslog(LOG_ERR, "%s: cannot read state for outlet %d.",
+                        	__FUNCTION__, outlet);
 			return (S_ACCESS);
 		}
 
@@ -683,19 +691,17 @@ apcmastersnmp_reset_req(Stonith * s, int request, const char *host)
 	        /* read reboot_duration of the port */
 	        if ((state = APC_read(ad->sptr, 
 			objname, ASN_INTEGER)) == NULL) {
-#ifdef APC_DEBUG
-		   syslog(LOG_DEBUG, 
-			"%s: cannot read outlet's reboot duration.",
-		       __FUNCTION__);
-#endif
+		   syslog(LOG_ERR, 
+			"%s: cannot read reboot duration for outlet %d.",
+		       __FUNCTION__, outlet);
 		   return (S_ACCESS);
 	        }
 	        if (num_outlets == 0) {
 		   /* save the inital value of the first port */
 		   reboot_duration = *state;
 	        } else if (reboot_duration != *state) {
-		  syslog(LOG_WARNING, "%s: Outlet %d has a different reboot duration!", 
-				__FUNCTION__, outlet);
+		  syslog(LOG_WARNING, "%s: outlet %d has a different reboot "
+                  	"duration!", __FUNCTION__, outlet);
 	    	  if (reboot_duration < *state)
 				reboot_duration = *state;
 	        }
@@ -706,30 +712,25 @@ apcmastersnmp_reset_req(Stonith * s, int request, const char *host)
 	}
     }
 #ifdef APC_DEBUG
-	    syslog(LOG_DEBUG, "%s: outlet: %i",
-		   __FUNCTION__, outlet);
+    syslog(LOG_DEBUG, "%s: outlet: %i.", __FUNCTION__, outlet);
 #endif
 
     /* host not found in outlet names */
     if (num_outlets < 1) {
-#ifdef APC_DEBUG
-	syslog(LOG_DEBUG, "%s: no active outlet '%s'.", __FUNCTION__, host);
-#endif
+	syslog(LOG_ERR, "%s: no active outlet for '%s'.", __FUNCTION__, host);
 	return (S_BADHOST);
     }
 
     /* Turn them all off */
 
-    for (outlet=outlets[0], i=0 ; i < num_outlets; i++, 
-		    outlet = outlets[i]) {
+    for (outlet=outlets[0], i=0 ; i < num_outlets; i++, outlet = outlets[i]) {
 	    /* prepare objname */
 	    snprintf(objname, MAX_STRING, OID_OUTLET_COMMAND_PENDING, outlet);
 
 	    /* are there pending commands ? */
 	    if ((state = APC_read(ad->sptr, objname, ASN_INTEGER)) == NULL) {
-#ifdef APC_DEBUG
-		syslog(LOG_DEBUG, "%s: cannot read outlet_pending.", __FUNCTION__);
-#endif
+		syslog(LOG_ERR, "%s: cannot read pending commands for "
+                    "outlet %d.", __FUNCTION__, outlet);
 		return (S_ACCESS);
 	    }
 
@@ -746,10 +747,8 @@ apcmastersnmp_reset_req(Stonith * s, int request, const char *host)
 
 	    /* send reboot cmd */
 	    if (!APC_write(ad->sptr, objname, 'i', value)) {
-#ifdef APC_DEBUG
-		syslog(LOG_DEBUG, "%s: cannot send reboot cmd for outlet %d.", 
+		syslog(LOG_ERR, "%s: cannot send reboot command for outlet %d.", 
 				__FUNCTION__, outlet);
-#endif
 		return (S_ACCESS);
 	    }
     }
@@ -768,10 +767,8 @@ apcmastersnmp_reset_req(Stonith * s, int request, const char *host)
 	    	/* get outlet's state */
 		
 		if ((state = APC_read(ad->sptr, objname, ASN_INTEGER)) == NULL) {
-#ifdef APC_DEBUG
-		    syslog(LOG_DEBUG, "%s: cannot read outlet_state of %d.",
-			   __FUNCTION__, outlets[0]);
-#endif
+		    syslog(LOG_ERR, "%s: cannot read state for outlet %d.",
+			   __FUNCTION__, outlet);
 		    return (S_ACCESS);
 		}
 
@@ -785,12 +782,14 @@ apcmastersnmp_reset_req(Stonith * s, int request, const char *host)
     
     if (bad_outlets == num_outlets) {
 	    /* reset failed */
-	    syslog(LOG_ERR, "%s: resetting host '%s' failed.", __FUNCTION__, host);
+	    syslog(LOG_ERR, "%s: resetting host '%s' failed.",
+                __FUNCTION__, host);
 	    return (S_RESETFAIL);
     } else {
 	    /* Not all outlets back on, but at least one; implies the node was */
 	    /* rebooted correctly */
-	    syslog(LOG_WARNING,"%s: Not all outlets came back online!", __FUNCTION__);
+	    syslog(LOG_WARNING,"%s: Not all outlets came back online!",
+                __FUNCTION__);
 	    return (S_OK); 
     }
 }
@@ -808,7 +807,7 @@ apcmastersnmp_set_config_file(Stonith * s, const char *configname)
     struct APCDevice *ad;
 
 #ifdef APC_DEBUG
-    syslog(LOG_DEBUG, "%s: called.", __FUNCTION__);
+    syslog(LOG_DEBUG, "%s: called, file '%s'.", __FUNCTION__, configname);
 #endif
 
     if (!ISAPCDEV(s)) {
@@ -819,7 +818,7 @@ apcmastersnmp_set_config_file(Stonith * s, const char *configname)
     ad = (struct APCDevice *) s->pinfo;
 
     if ((cfgfile = fopen(configname, "r")) == NULL) {
-	syslog(LOG_ERR, "Cannot open %s", configname);
+	syslog(LOG_ERR, "cannot open %s.", configname);
 	return (S_BADCONFIG);
     }
 
@@ -841,11 +840,7 @@ apcmastersnmp_set_config_info(Stonith * s, const char *info)
     struct APCDevice *ad;
 
 #ifdef APC_DEBUG
-    syslog(LOG_DEBUG, "%s: called.", __FUNCTION__);
-#endif
-
-#ifdef APC_DEBUG
-    syslog(LOG_DEBUG, "%s: info: '%s'.", __FUNCTION__, info);
+    syslog(LOG_DEBUG, "%s: called, info '%s'.", __FUNCTION__, info);
 #endif
 
     if (!ISAPCDEV(s)) {
