@@ -1,3 +1,4 @@
+/* $Id: baytech.c,v 1.13.2.1 2004/04/20 08:50:06 alan Exp $ */
 /*
  *	Stonith module for BayTech Remote Power Controllers (RPC-x devices)
  *
@@ -28,6 +29,7 @@
 #include <syslog.h>
 #include <libintl.h>
 #include <sys/wait.h>
+#include <glib.h>
 
 #include <stonith/stonith.h>
 #define PIL_PLUGINTYPE          STONITH_TYPE
@@ -96,6 +98,7 @@ static void*			interfprivate;
 
 #define LOG		PluginImports->log
 #define MALLOC		PluginImports->alloc
+#define STRDUP  	PluginImports->mstrdup
 #define FREE		PluginImports->mfree
 #define EXPECT_TOK	OurImports->ExpectToken
 #define STARTPROC	OurImports->StartProcess
@@ -152,29 +155,26 @@ struct BayTech {
 };
 
 struct BayTechModelInfo {
-		const char *	type;		/* Baytech model info */
-		int		socklen;	/* Length of socket name string */
-		struct Etoken *	expect;		/* Expect string before outlet list */
+	const char *	type;		/* Baytech model info */
+	size_t		socklen;	/* Length of socket name string */
+	struct Etoken *	expect;		/* Expect string before outlet list */
 };
 
+static int		parse_socket_line(struct BayTech*,const char *
+,			int *, char *);
+
 static const char * BTid = "BayTech-Stonith";
 static const char * NOTbtid = "Hey, dummy this has been destroyed (BayTech)";
 
 #define	ISBAYTECH(i)	(((i)!= NULL && (i)->pinfo != NULL)	\
 	&& ((struct BayTech *)(i->pinfo))->BTid == BTid)
 
-#define	ISCONFIGED(i)	(ISBAYTECH(i) && ((struct BayTech *)(i->pinfo))->config)
-
-#ifndef MALLOC
-#	define	MALLOC	malloc
-#endif
-#ifndef FREE
-#	define	FREE	free
-#endif
 #ifndef MALLOCT
-#	define     MALLOCT(t)      ((t *)(MALLOC(sizeof(t))))
+#	define MALLOCT(t) ((t *)(MALLOC(sizeof(t))))
 #endif
 
+#define	ISCONFIGED(i)	(ISBAYTECH(i) && ((struct BayTech *)(i->pinfo))->config)
+
 #define WHITESPACE	" \t\n\r\f"
 
 #define	REPLSTR(s,v)	{					\
@@ -182,11 +182,9 @@ static const char * NOTbtid = "Hey, dummy this has been destroyed (BayTech)";
 				FREE(s);			\
 				(s)=NULL;			\
 			}					\
-			(s) = MALLOC(strlen(v)+1);		\
+			(s) = STRDUP(v);			\
 			if ((s) == NULL) {			\
 				syslog(LOG_ERR, _("out of memory"));\
-			}else{					\
-				strcpy((s),(v));		\
 			}					\
 			}
 
@@ -221,16 +219,16 @@ static struct Etoken CRNL[] =		{ {"\n\r",0,0},{"\r\n",0,0},{NULL,0,0}};
 
 /* We may get a notice about rebooting, or a request for confirmation */
 static struct Etoken Rebooting[] =	{ {"ebooting selected outlet", 0, 0}
-				,	{"(Y/N)>", 1, 0}
-				,	{"already off.", 2, 0}
-				,	{NULL,0,0}};
+					,	{"(Y/N)>", 1, 0}
+					,	{"already off.", 2, 0}
+					,	{NULL,0,0}};
 
 
 static struct BayTechModelInfo ModelInfo [] = {
-		{"RPC-5", 18, Temp},	/* This first model will be the default */
-		{"RPC-3", 10, Break},	
-		{"RPC-3A", 10, Break},
-		{NULL, 0, NULL},
+	{"RPC-5", 18, Temp},	/* This first model will be the default */
+	{"RPC-3", 10, Break},	
+	{"RPC-3A", 10, Break},
+	{NULL, 0, NULL},
 };
 
 static int	RPCLookFor(struct BayTech* bt, struct Etoken * tlist, int timeout);
@@ -579,13 +577,13 @@ RPC_onoff(struct BayTech* bt, int unitnum, const char * unitid, int req)
 
 	/* Expect "RPC->x "... or "(Y/N)" (if confirmation turned on) */
 
-	if (RPCLookFor(bt, RPC, 5) == 1) {
+	if (RPCLookFor(bt, RPC, 10) == 1) {
 		/* They've turned on that annoying command confirmation :-( */
 		SEND("Y\r");
-		EXPECT(RPC, 5);
+		EXPECT(RPC, 10);
 	}
 
-	EXPECT(GTSign, 5);
+	EXPECT(GTSign, 10);
 
 	/* All Right!  Command done. Life is Good! */
 	syslog(LOG_NOTICE, _("Power to host %s turned %s."), unitid, onoff);
@@ -606,11 +604,8 @@ RPCNametoOutlet(struct BayTech* bt, const char * name)
 	int	sockno;
 	char	sockname[32];
 	int	ret = -1;
-	char	format[32];
 
 
-	snprintf(format, sizeof(format), "%%7d       %%%dc"
-	,	bt->modelinfo->socklen);
 
 	/* Verify that we're in the top-level menu */
 	SEND("\r");
@@ -639,26 +634,30 @@ RPCNametoOutlet(struct BayTech* bt, const char * name)
 	/* Looks Good!  Parse the status output */
 
 	do {
+		char *	last;
 		NameMapping[0] = EOS;
 		SNARF(NameMapping, 5);
-		if (sscanf(NameMapping, format, &sockno, sockname) == 2) {
 
-			char *	last = sockname+bt->modelinfo->socklen;
-			*last = EOS;
-			--last;
+		if (!parse_socket_line(bt, NameMapping, &sockno, sockname)) {
+			continue;
+		}
 
-			/* Strip off trailing blanks */
-			for(; last > sockname; --last) {
-				if (*last == ' ') {
-					*last = EOS;
-				}else{
-					break;
-				}
-			}
-			if (strcmp(name, sockname) == 0) {
-				ret = sockno;
+		last = sockname+bt->modelinfo->socklen;
+		*last = EOS;
+		--last;
+
+		/* Strip off trailing blanks */
+		for(; last > sockname; --last) {
+			if (*last == ' ') {
+				*last = EOS;
+			}else{
+				break;
 			}
 		}
+		g_strdown(sockname);
+		if (strcmp(name, sockname) == 0) {
+			ret = sockno;
+		}
 	} while (strlen(NameMapping) > 2 && ret < 0);
 
 	/* Pop back out to the top level menu */
@@ -709,7 +708,6 @@ baytech_hostlist(Stonith  *s)
 	unsigned int	numnames = 0;
 	char **		ret = NULL;
 	struct BayTech*	bt;
-	char		format[32];
 
 
 	if (!ISBAYTECH(s)) {
@@ -724,8 +722,6 @@ baytech_hostlist(Stonith  *s)
 	bt = (struct BayTech*) s->pinfo;
 
 
-	snprintf(format, sizeof(format), "%%7d       %%%dc"
-	,	bt->modelinfo->socklen);
 
 	if (RPCRobustLogin(bt) != S_OK) {
 		syslog(LOG_ERR, _("Cannot log into " DEVICE "."));
@@ -761,35 +757,40 @@ baytech_hostlist(Stonith  *s)
 	do {
 		int	sockno;
 		char	sockname[64];
+		char *	last;
+		char *	nm;
+
 		NameMapping[0] = EOS;
+
 		NULLSNARF(NameMapping, 5);
-		if (sscanf(NameMapping, format, &sockno, sockname) == 2) {
 
-			char *	last = sockname+bt->modelinfo->socklen;
-			char *	nm;
-			*last = EOS;
-			--last;
+		if (!parse_socket_line(bt, NameMapping, &sockno, sockname)) {
+			continue;
+		}
 
-			/* Strip off trailing blanks */
-			for(; last > sockname; --last) {
-				if (*last == ' ') {
-					*last = EOS;
-				}else{
-					break;
-				}
-			}
-			if (numnames >= DIMOF(NameList)-1) {
+		last = sockname+bt->modelinfo->socklen;
+		*last = EOS;
+		--last;
+
+		/* Strip off trailing blanks */
+		for(; last > sockname; --last) {
+			if (*last == ' ') {
+				*last = EOS;
+			}else{
 				break;
 			}
-			if ((nm = (char*)MALLOC(strlen(sockname)+1)) == NULL) {
-				syslog(LOG_ERR, "out of memory");
-				return(NULL);
-			}
-			strcpy(nm, sockname);
-			NameList[numnames] = nm;
-			++numnames;
-			NameList[numnames] = NULL;
 		}
+		if (numnames >= DIMOF(NameList)-1) {
+			break;
+		}
+		if ((nm = (char*)STRDUP(sockname)) == NULL) {
+			syslog(LOG_ERR, "out of memory");
+			return(NULL);
+		}
+		g_strdown(nm);
+		NameList[numnames] = nm;
+		++numnames;
+		NameList[numnames] = NULL;
 	} while (strlen(NameMapping) > 2);
 
 	/* Pop back out to the top level menu */
@@ -841,27 +842,24 @@ RPC_parse_config_info(struct BayTech* bt, const char * info)
 	if (sscanf(info, "%s %s %[^\n\r\t]", dev, user, passwd) == 3
 	&&	strlen(passwd) > 1) {
 
-		if ((bt->device = (char *)MALLOC(strlen(dev)+1)) == NULL) {
+		if ((bt->device = STRDUP(dev)) == NULL) {
 			syslog(LOG_ERR, "out of memory");
 			return(S_OOPS);
 		}
-		if ((bt->user = (char *)MALLOC(strlen(user)+1)) == NULL) {
-			free(bt->device);
+		if ((bt->user = STRDUP(user)) == NULL) {
+			FREE(bt->device);
 			bt->device=NULL;
 			syslog(LOG_ERR, "out of memory");
 			return(S_OOPS);
 		}
-		if ((bt->passwd = (char *)MALLOC(strlen(passwd)+1)) == NULL) {
-			free(bt->user);
+		if ((bt->passwd = STRDUP(passwd)) == NULL) {
+			FREE(bt->user);
 			bt->user=NULL;
-			free(bt->device);
+			FREE(bt->device);
 			bt->device=NULL;
 			syslog(LOG_ERR, "out of memory");
 			return(S_OOPS);
 		}
-		strcpy(bt->device, dev);
-		strcpy(bt->user, user);
-		strcpy(bt->passwd, passwd);
 		bt->config = 1;
 		return(S_OK);
 	}
@@ -1121,3 +1119,31 @@ baytech_new(void)
 
 	return((void *)bt);
 }
+
+static int
+parse_socket_line(struct BayTech * bt,	const char *NameMapping
+,	int *sockno, char *sockname)
+{
+#if 0
+	char format[64];
+	snprintf(format, sizeof(format), "%%7d       %%%dc"
+	,	bt->modelinfo->socklen);
+	/* 7 digits, 7 blanks, then 'socklen' characters */
+	/* [0-6]: digits, NameMapping[13] begins the sockname */
+	/* NameMapping strlen must be >= socklen + 14 */
+
+	if (sscanf(NameMapping, format, sockno, sockname) != 2) {
+		return FALSE;
+	}
+#else
+#	define	OFFSET 14
+
+	if (sscanf(NameMapping, "%7d", sockno) != 1
+	||	strlen(NameMapping) < OFFSET+bt->modelinfo->socklen) {
+		return FALSE;
+	}
+	strncpy(sockname, NameMapping+OFFSET, bt->modelinfo->socklen);
+	sockname[bt->modelinfo->socklen] = EOS;
+#endif
+	return TRUE;
+}
