@@ -1,4 +1,4 @@
-/* $Id: meatware.c,v 1.8.2.1 2004/04/20 08:50:06 alan Exp $ */
+/* $Id: meatware.c,v 1.8.2.3 2005/08/19 16:19:01 sunjd Exp $ */
 /*
  * Stonith module for Human Operator Stonith device
  *
@@ -308,7 +308,7 @@ static int
 meatware_reset_req(Stonith * s, int request, const char * host)
 {
 	int fd, rc;
-	const char *	meatpipe_pr = "/tmp/.meatware"; /* if you intend to
+	const char *	meatpipe_pr = HA_VARRUNDIR"/meatware"; /* if you intend to
 							change this, modify
 							meatclient.c as well */
 
