@@ -1,3 +1,4 @@
+/* $Id: nw_rpc100s.c,v 1.10.2.1 2004/04/20 08:50:06 alan Exp $ */
 /*
  *	Stonith module for Night/Ware RPC100S 
  *
@@ -38,6 +39,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <glib.h>
 
 #include <stonith/stonith.h>
 #define PIL_PLUGINTYPE          STONITH_TYPE
@@ -104,6 +106,7 @@ static void*			interfprivate;
 
 #define LOG		PluginImports->log
 #define MALLOC		PluginImports->alloc
+#define STRDUP  	PluginImports->mstrdup
 #define FREE		PluginImports->mfree
 #define EXPECT_TOK	OurImports->ExpectToken
 #define STARTPROC	OurImports->StartProcess
@@ -223,12 +226,6 @@ static int gbl_debug = DEBUG;
 
 #define	ISCONFIGED(i)	(ISNWRPC100S(i) && ((struct NW_RPC100S *)(i->pinfo))->config)
 
-#ifndef MALLOC
-#	define	MALLOC	malloc
-#endif
-#ifndef FREE
-#	define	FREE	free
-#endif
 #ifndef MALLOCT
 #	define     MALLOCT(t)      ((t *)(MALLOC(sizeof(t)))) 
 #endif
@@ -240,11 +237,9 @@ static int gbl_debug = DEBUG;
 				FREE(s);			\
 				(s)=NULL;			\
 			}					\
-			(s) = MALLOC(strlen(v)+1);		\
+			(s) = STRDUP(v);			\
 			if ((s) == NULL) {			\
 				syslog(LOG_ERR, _("out of memory"));\
-			}else{					\
-				strcpy((s),(v));		\
 			}					\
 			}
 
@@ -537,8 +532,15 @@ nw_rpc100s_hostlist(Stonith  *s)
 	if (ret == NULL) {
 		syslog(LOG_ERR, "out of memory");
 	} else {
-		ret[0]=strdup(ctx->node);
 		ret[1]=NULL;
+		ret[0]=STRDUP(ctx->node);
+		if (ret[0] == NULL) {
+			syslog(LOG_ERR, "out of memory");
+			FREE(ret);
+			ret = NULL;
+		} else {
+			g_strdown(ret[0]);
+		}
 	}
 
 	return(ret);
@@ -604,7 +606,7 @@ RPC_parse_config_info(struct NW_RPC100S* ctx, const char * info)
 	   duration of this function.
 	*/
 
-	copy = strdup(info);
+	copy = STRDUP(info);
 	if (!copy) {
 		syslog(LOG_ERR, "out of memory");
 		return S_OOPS;
@@ -618,7 +620,7 @@ RPC_parse_config_info(struct NW_RPC100S* ctx, const char * info)
 		goto token_error;		
 	}
 
-	ctx->device = strdup(token);
+	ctx->device = STRDUP(token);
 	if (!ctx->device) {
 		syslog(LOG_ERR, "out of memory");
 		goto token_error;
@@ -633,7 +635,7 @@ RPC_parse_config_info(struct NW_RPC100S* ctx, const char * info)
 		goto token_error;		
 	}
 
-	ctx->node = strdup(token);
+	ctx->node = STRDUP(token);
 	if (!ctx->node) {
 		syslog(LOG_ERR, "out of memory");
 		goto token_error;
@@ -643,12 +645,12 @@ RPC_parse_config_info(struct NW_RPC100S* ctx, const char * info)
 	/* free our private copy of the string we've been destructively 
 	   parsing with strtok()
 	*/
-	free(copy);
+	FREE(copy);
 	ctx->config = 1;
 	return S_OK;
 
 token_error:
-	free(copy);
+	FREE(copy);
 	return(S_BADCONFIG);
 }
 
@@ -755,10 +757,18 @@ RPCDisconnect(struct NW_RPC100S * ctx)
 static int
 RPCNametoOutlet ( struct NW_RPC100S * ctx, const char * host )
 {
+	char *shost;
+	int rc = -1;
+	
+	if ( (shost = strdup(host)) == NULL) {
+		syslog(LOG_ERR, "strdup failed in RPCNametoOutlet");
+		return -1;
+	}
 	if (!strcmp(ctx->node, host))
-		return 0;
+		rc = 0;
 
-	return -1;
+	free(shost);
+	return rc;
 }
 
 
@@ -837,7 +847,7 @@ nw_rpc100s_set_config_file(Stonith* s, const char * configname)
 {
 	FILE *	cfgfile;
 
-	char	RPCid[256];
+	char	cfgline[256];
 
 	struct NW_RPC100S*	ctx;
 
@@ -852,9 +862,9 @@ nw_rpc100s_set_config_file(Stonith* s, const char * configname)
 		return(S_BADCONFIG);
 	}
 
-	while (fgets(RPCid, sizeof(RPCid), cfgfile) != NULL){
+	while (fgets(cfgline, sizeof(cfgline), cfgfile) != NULL){
 
-		RPC_parse_config_info(ctx, RPCid);
+		RPC_parse_config_info(ctx, cfgline);
 	}
 	return(S_BADCONFIG);
 }
