@@ -1,4 +1,4 @@
-/* $Id: meatclient.c,v 1.4.4.1 2004/04/20 09:08:51 lars Exp $ */
+/* $Id: meatclient.c,v 1.4.4.4 2005/08/22 07:12:54 sunjd Exp $ */
 /*
  * Stonith client for Human Operator Stonith device
  *
@@ -25,6 +25,7 @@
  */
 
 #include <portability.h>
+#include <config.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -55,7 +56,7 @@ int
 main(int argc, char** argv)
 {
 	char *		cmdname;
-	const char *	meatpipe_pr = "/tmp/.meatware";	/* if you intend to
+	const char *	meatpipe_pr = HA_VARRUNDIR"/meatware";	/* if you intend to
 							 change this, modify
 							 meatware.c as well */
 	char *		opthost = NULL;
