@@ -1,4 +1,4 @@
-/* $Id: ccmlib.h,v 1.13.2.2 2004/09/11 06:36:34 msoffen Exp $ */
+/* $Id: ccmlib.h,v 1.13.2.3 2004/12/01 00:11:28 yixiong Exp $ */
 /*
  * ccmlib.h: internal definations for ccm library files.
  *
@@ -21,6 +21,7 @@
  */
 #ifndef __CCMLIB_H_
 #define __CCMLIB_H_
+#include <portability.h>
 #include <errno.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -39,7 +40,7 @@ void oc_ev_special(const oc_ev_t *, oc_ev_class_t , int );
 
 #define NODEIDSIZE 255 /* if this value is changed change it 
 			  	also in ccm.h */
-#define CCMFIFO "/var/lib/heartbeat/ccm/ccm" /* if this value
+#define CCMFIFO HA_VARLIBDIR "/heartbeat/ccm/ccm" /* if this value
 			is changed change it also in ccm.h */
 
 size_t strnlen(const char *, size_t); /*TOBEDONE*/
