@@ -1,3 +1,4 @@
+/* $Id: strerror.c,v 1.4.2.1 2004/04/20 05:05:36 alan Exp $ */
 /*
  * Copyright (C) 2002 Alan Robertson <alanr@unix.sh>
  * This software licensed under the GNU LGPL.
@@ -21,7 +22,7 @@
 #include <stdio.h>
 extern const char *	sys_err[];
 extern int	sys_nerr;
-const char *
+char *
 strerror(int errnum)
 {
 	static	char  whaterr[32];
