@@ -32,6 +32,7 @@
 #include "hb_api.h"
 #include "heartbeat.h"
 #include "clplumbing/cl_log.h"
+#include "clplumbing/coredumps.h"
 
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -59,14 +60,14 @@
 
 static unsigned long hbInitialized = 0;
 static ll_cluster_t * hb = NULL; /* heartbeat handle */
-static const char * myid = NULL; /* my node id */
+static char * myid = NULL; /* my node id */
 static SaClmHandleT clm = 0;
 static unsigned long clmInitialized = 0;
 
-static GArray * gNodeTable = NULL;
-static GArray * gIFTable = NULL;
-static GArray * gMembershipTable = NULL;
-static GArray * gResourceTable = NULL;
+static GPtrArray * gNodeTable = NULL;
+static GPtrArray * gIFTable = NULL;
+static GPtrArray * gMembershipTable = NULL;
+static GPtrArray * gResourceTable = NULL;
 
 static int keep_running;
 
@@ -84,20 +85,95 @@ int init_storage(void);
 
 void free_storage(void);
 
-int walk_node_table(void);
-int walk_if_table(void);
+int walk_nodetable(void);
+int walk_iftable(void);
 
 int nodestatus_trap(const char * node, const char * status);
 int ifstatus_trap(const char * node, const char * lnk, const char * status);
 int membership_trap(const char * node, SaClmClusterChangesT status);
+int hbagent_trap(int online, const char * node);
+
+int ping_membership(int * mem_fd);
+
+uint32_t get_status_value(const char * status, const char * * status_array, uint32_t * value_array);
+
+const char * NODE_STATUS [] = 
+{
+	"",
+	INITSTATUS,
+	UPSTATUS,
+	ACTIVESTATUS,
+	DEADSTATUS,
+	""
+};
+
+static uint32_t NODE_STATUS_VALUE[] = 
+{	
+	LHANODESTATUS_UNKNOWN, 
+	LHANODESTATUS_INIT,
+	LHANODESTATUS_UP,
+	LHANODESTATUS_ACTIVE,
+	LHANODESTATUS_DEAD,
+};
+
+const char * NODE_TYPE [] =
+{
+	UNKNOWNNODE,
+	NORMALNODE,
+	PINGNODE,
+	""
+};
+
+static uint32_t NODE_TYPE_VALUE[] =
+{
+	LHANODETYPE_UNKNOWN,
+	LHANODETYPE_NORMAL,
+	LHANODETYPE_PING
+};
+
+const char * IF_STATUS[] = 
+{
+	"",
+	LINKUP,
+	DEADSTATUS,
+	""
+};
+
+static uint32_t IF_STATUS_VALUE[] = 
+{
+	LHAIFSTATUS_UNKNOWN,
+	LHAIFSTATUS_UP,
+	LHAIFSTATUS_DOWN
+};
 
 static RETSIGTYPE
 stop_server(int a) {
     keep_running = 0;
 }
 
+uint32_t 
+get_status_value(const char * status, const char * * status_array, uint32_t * value_array)
+{
+	int i = 1;
+	int found = 0;
+
+	while (strlen(status_array[i])) {
+		if (strncmp(status, status_array[i], 
+					strlen(status_array[i])) == 0) {
+			found = 1;
+			break;
+		}
+		i++;
+	}
+
+	if (found)
+		return value_array[i];
+	else 
+		return value_array[0];
+}
+
 int 
-get_int_value(lha_group_t group, lha_attribute_t attr, size_t index, int * value)
+get_int_value(lha_group_t group, lha_attribute_t attr, size_t index, uint32_t * value)
 {
 	switch (group) {
 		case LHA_CLUSTERINFO: 
@@ -135,11 +211,12 @@ get_str_value(lha_group_t group, lha_attribute_t attr, size_t index, char * * va
 }
 
 int
-clusterinfo_get_int_value(lha_attribute_t attr, size_t index, int32_t * value)
+clusterinfo_get_int_value(lha_attribute_t attr, size_t index, uint32_t * value)
 {
     int i;
-    const char * status;
+    uint32_t status;
     size_t count;
+    const struct hb_nodeinfo * node;
 
     *value = 0;
 
@@ -158,8 +235,9 @@ clusterinfo_get_int_value(lha_attribute_t attr, size_t index, int32_t * value)
 	    count = 0;
 
 	    for (i = 0; i < gNodeTable->len; i++) {
-		status = g_array_index(gNodeTable, struct hb_nodeinfo, i).status;
-		if (strcmp(status, DEADSTATUS) != 0) {
+		status = ((struct hb_nodeinfo *) g_ptr_array_index(gNodeTable, i))->status;
+		if (status != LHANODESTATUS_DEAD &&
+			status != LHANODESTATUS_UNKNOWN ) {
 		    count++;
 		}
 	    }
@@ -167,6 +245,22 @@ clusterinfo_get_int_value(lha_attribute_t attr, size_t index, int32_t * value)
 	    *value = count;
 	    break;
 
+	case RESOURCE_GROUP_COUNT:
+	    *value = gResourceTable->len;
+
+	    break;
+
+	case CURRENT_NODE_ID:
+	    for (i = 0; i < gNodeTable->len; i++) {
+		node = (struct hb_nodeinfo *) g_ptr_array_index(gNodeTable, i);
+		if (strcmp(node->name, myid) == 0) {
+		    *value = node->id;
+		    break;
+		}
+	    }
+
+	    break;
+
 	default:
 	    return HA_FAIL;
     }
@@ -180,23 +274,24 @@ hbconfig_get_str_value(const char * attr, char * * value)
     char * ret;
     static char err[] = "N/A";
 
-    *value  = err;
-
     if ((ret = hb->llc_ops->get_parameter(hb, attr)) == NULL) {
-	cl_log(LOG_ERR, "getting parameter [%s] error.", attr);
+	/*
+	cl_log(LOG_INFO, "getting parameter [%s] error.", attr);
 	cl_log(LOG_INFO, "reason: %s.", hb->llc_ops->errmsg(hb));
+	*/
 
 	/* we have to return HA_OK here otherwise the 
 	   agent code would not progress */
+	*value  = ha_strdup(err);
 	return HA_OK;
     };
 
-    *value = ret;
+    *value = ha_strdup(ret);
 
     return HA_OK;
 }
 
-GArray *
+GPtrArray *
 get_hb_info(lha_group_t group)
 {
     switch (group) {
@@ -224,7 +319,7 @@ NodeStatus(const char * node, const char * status, void * private)
 {
         cl_log(LOG_NOTICE, "Status update: Node %s now has status %s\n"
         ,       node, status);
-	walk_node_table();
+	walk_nodetable();
 
 	nodestatus_trap(node, status);
 }
@@ -235,7 +330,7 @@ LinkStatus(const char * node, const char * lnk, const char * status
 {
         cl_log(LOG_NOTICE, "Link Status update: Link %s/%s now has status %s\n"
         ,       node, lnk, status);
-	walk_if_table();
+	walk_iftable();
 
 	ifstatus_trap(node, lnk, status);
 }
@@ -243,11 +338,10 @@ LinkStatus(const char * node, const char * lnk, const char * status
 int
 init_storage(void)
 {
-	gNodeTable = g_array_new(TRUE, TRUE, sizeof (struct hb_nodeinfo));
-	gIFTable = g_array_new(TRUE, TRUE, sizeof (struct hb_ifinfo));
-	gResourceTable = g_array_new(TRUE, TRUE, sizeof (struct hb_rsinfo));
-	gMembershipTable = g_array_new(TRUE, TRUE, 
-			sizeof (SaClmClusterNotificationT));
+	gNodeTable = g_ptr_array_new();
+	gIFTable = g_ptr_array_new();
+	gResourceTable = g_ptr_array_new();
+	gMembershipTable = g_ptr_array_new();
 
 	if (!gNodeTable || !gIFTable || !gResourceTable || !gMembershipTable){
 	    	cl_log(LOG_ERR, "Storage allocation failure.  Out of Memory.");
@@ -267,13 +361,10 @@ free_nodetable(void)
 
 	while (gNodeTable->len) {
 
-	    	node = &g_array_index(gNodeTable, struct hb_nodeinfo, 0);
+		node = (struct hb_nodeinfo *) g_ptr_array_remove_index_fast(gNodeTable, 0);
 
 		free(node->name);
-		free(node->type);
-		free(node->status);
-
-		gNodeTable = g_array_remove_index_fast(gNodeTable, 0);
+		ha_free(node);
 	}
 
 	return;
@@ -288,13 +379,11 @@ free_iftable(void)
 		return;
 
 	while (gIFTable->len) {
-		interface = &g_array_index(gIFTable, struct hb_ifinfo, 0);
+		interface = (struct hb_ifinfo *) g_ptr_array_remove_index_fast(gIFTable, 0);
 
 		free(interface->name);
 		free(interface->node);
-		free(interface->status);
-
-		gIFTable = g_array_remove_index_fast(gIFTable, 0);
+		ha_free(interface);
 	}
 
 	return;
@@ -310,13 +399,11 @@ free_resourcetable(void)
 
 	while (gResourceTable->len) {
 
-		resource = & g_array_index(gResourceTable, 
-			struct hb_rsinfo, 0);
+		resource = (struct hb_rsinfo *) g_ptr_array_remove_index_fast(gResourceTable, 0);
 
-		free(resource->master);
 		free(resource->resource);
+		ha_free(resource);
 
-		gResourceTable = g_array_remove_index_fast(gResourceTable, 0);
 	}
 
 	return;
@@ -333,8 +420,7 @@ free_membershiptable(void)
 	   any memory. */
 
 	while (gMembershipTable->len) {
-		gMembershipTable = 
-	    		g_array_remove_index_fast(gMembershipTable, 0);
+	    	g_ptr_array_remove_index_fast(gMembershipTable, 0);
 	}
 
 	return;
@@ -344,19 +430,19 @@ void
 free_storage(void)
 {
     	free_nodetable();
-	g_array_free(gNodeTable, 1);
+	g_ptr_array_free(gNodeTable, 0);
 	gNodeTable = NULL;
 
 	free_iftable();
-	g_array_free(gIFTable, 1);
+	g_ptr_array_free(gIFTable, 0);
 	gIFTable = NULL;
 
 	free_resourcetable();
-	g_array_free(gResourceTable, 1);
+	g_ptr_array_free(gResourceTable, 0);
 	gResourceTable = NULL;
 
 	free_membershiptable();
-	g_array_free(gMembershipTable, 1);
+	g_ptr_array_free(gMembershipTable, 0);
 	gResourceTable = NULL;
 
 }
@@ -364,6 +450,7 @@ free_storage(void)
 int
 init_heartbeat(void)
 {
+	const char * parameter;
 	hb = NULL;
 
 	cl_log_set_entity("lha-snmpagent");
@@ -371,8 +458,8 @@ init_heartbeat(void)
 
 	hb = ll_cluster_new("heartbeat");
 
-	cl_log(LOG_DEBUG, "PID=%ld\n", (long)getpid());
-	cl_log(LOG_DEBUG, "Signing in with heartbeat\n");
+	cl_log(LOG_DEBUG, "PID=%ld", (long)getpid());
+	cl_log(LOG_DEBUG, "Signing in with heartbeat");
 
 	if (hb->llc_ops->signon(hb, LHAAGENTID)!= HA_OK) {
 		cl_log(LOG_ERR, "Cannot sign on with heartbeat\n");
@@ -380,7 +467,14 @@ init_heartbeat(void)
 		return HA_FAIL;
 	}
 
-	if (NULL == (myid = hb->llc_ops->get_mynodeid(hb))) {
+	/* See if we should drop cores somewhere odd... */
+	parameter = hb->llc_ops->get_parameter(hb, KEY_COREROOTDIR);
+	if (parameter) {
+		cl_set_corerootdir(parameter);
+	}
+	cl_cdtocoredir();
+
+	if (NULL == (myid = ha_strdup(hb->llc_ops->get_mynodeid(hb)))) {
 		cl_log(LOG_ERR, "Cannot get mynodeid\n");
 		cl_log(LOG_ERR, "REASON: %s\n", hb->llc_ops->errmsg(hb));
 		return HA_FAIL;
@@ -400,7 +494,7 @@ init_heartbeat(void)
 
 #if 1
 	/* walk the node table */
-	if ( HA_OK != walk_node_table() || HA_OK != walk_if_table() ) {
+	if ( HA_OK != walk_nodetable() || HA_OK != walk_iftable() ) {
 		return HA_FAIL;
 	}
 #endif
@@ -428,86 +522,118 @@ get_heartbeat_fd(void)
 }
 
 int
-walk_node_table(void)
+walk_nodetable(void)
 {
 	const char *name, *type, *status;
-	struct hb_nodeinfo node;
+	struct hb_nodeinfo * node;
+	size_t id = 0;
+	uuid_t uuid;
 
 	if (gNodeTable) {
 		free_nodetable();
 	}
 
 	if (hb->llc_ops->init_nodewalk(hb) != HA_OK) {
-		cl_log(LOG_ERR, "Cannot start node walk\n");
-		cl_log(LOG_ERR, "REASON: %s\n", hb->llc_ops->errmsg(hb));
+		cl_log(LOG_ERR, "Cannot start node walk");
+		cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
 		return HA_FAIL;
 	}
 	while((name = hb->llc_ops->nextnode(hb))!= NULL) {
+		id++;
+
 		status = hb->llc_ops->node_status(hb, name);
-
-		cl_log(LOG_DEBUG, "Cluster node: %s: status: %s\n", name 
-		,	status);
-
 		type = hb->llc_ops->node_type(hb, name);
 
-		cl_log(LOG_DEBUG, "Cluster node: %s: type: %s\n", name 
-		,	type);
+		cl_log(LOG_INFO, "node %ld: %s, type: %s, status: %s"
+		,	(long)id, name 
+		,	type, status);
 
-		node.name =  g_strdup(name);
-		node.type =  g_strdup(type);
-		node.status = g_strdup(status);
-		g_array_append_val(gNodeTable, node); 
+		node = (struct hb_nodeinfo *) ha_malloc(sizeof(struct hb_nodeinfo));
+		if (!node) {
+			cl_log(LOG_CRIT, "malloc failed for node info.");
+			return HA_FAIL;
+		}
+
+		node->name =  g_strdup(name);
+		node->ifcount = 0;
+		node->id = id;
+
+		memset(uuid, 0, sizeof(uuid_t));
+
+#ifdef HAVE_NEW_HB_API
+		/* the get_uuid_by_name is not available for STABLE_1_2 branch. */
+		if (hb->llc_ops->get_uuid_by_name(hb, name, uuid) == HA_FAIL) {
+			cl_log(LOG_DEBUG, "Cannot get the uuid for node: %s", name);
+		}
+#endif /* HAVE_NEW_HB_API */
+		memcpy(node->uuid, uuid, sizeof(uuid_t));
+
+		node->type = get_status_value(type, NODE_TYPE, NODE_TYPE_VALUE);
+		node->status = get_status_value(status, NODE_STATUS,
+				NODE_STATUS_VALUE);
+
+		g_ptr_array_add(gNodeTable, (gpointer *) node); 
 	}
 	if (hb->llc_ops->end_nodewalk(hb) != HA_OK) {
-		cl_log(LOG_ERR, "Cannot end node walk\n");
-		cl_log(LOG_ERR, "REASON: %s\n", hb->llc_ops->errmsg(hb));
+		cl_log(LOG_ERR, "Cannot end node walk");
+		cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
 		return HA_FAIL;
 	}
 	return HA_OK;
 }
 
 int
-walk_if_table(void)
+walk_iftable(void)
 {
 	const char *name, * status;
 	struct hb_nodeinfo * node;
-	struct hb_ifinfo interface;
-	int i, ifcount;
+	struct hb_ifinfo * interface;
+	int i; 
+	size_t ifcount;
 
 	if (gIFTable) {
 		free_iftable();
 	}
 
 	for (i = 0; i < gNodeTable->len; i++) {
-		node = &g_array_index(gNodeTable, struct hb_nodeinfo, i);
+		node = (struct hb_nodeinfo *) g_ptr_array_index(gNodeTable, i);
 		ifcount = 0;
 
 		if (hb->llc_ops->init_ifwalk(hb, node->name) != HA_OK) {
-			cl_log(LOG_ERR, "Cannot start if walk\n");
-			cl_log(LOG_ERR, "REASON: %s\n", hb->llc_ops->errmsg(hb));
+			cl_log(LOG_ERR, "Cannot start if walk");
+			cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
 			return HA_FAIL;
 		}
 
 		while((name = hb->llc_ops->nextif(hb))!=NULL) {
 			status = hb->llc_ops->if_status(hb, node->name, name);
 
-			cl_log(LOG_DEBUG, "node interface: %s: status: %s\n",
-					name,	status);
+			cl_log(LOG_INFO, "node: %s, interface: %s, status: %s",
+					node->name, name, status);
 
-			interface.name = g_strdup(name);
-			interface.node = g_strdup(node->name);
-			interface.status = g_strdup(status);
-			interface.nodeid= i;
-			interface.id = ifcount++;
-			g_array_append_val(gIFTable, interface);
+			interface = (struct hb_ifinfo *) ha_malloc(sizeof(struct hb_ifinfo));
+			if (!interface) {
+				cl_log(LOG_CRIT, "malloc failed for if info.");
+				return HA_FAIL;
+			}
+
+			interface->name = g_strdup(name);
+			interface->node = g_strdup(node->name);
+			interface->nodeid = node->id;
+			interface->id = ++ifcount;
+			interface->status = get_status_value(status,
+					IF_STATUS, IF_STATUS_VALUE);
+
+			g_ptr_array_add(gIFTable, (gpointer *) interface);
 		}
 
+		node->ifcount = ifcount;
+
 		if (hb->llc_ops->end_ifwalk(hb) != HA_OK) {
-			cl_log(LOG_ERR, "Cannot end if walk.\n");
-			cl_log(LOG_ERR, "REASON: %s\n", hb->llc_ops->errmsg(hb));
+			cl_log(LOG_ERR, "Cannot end if walk.");
+			cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
 			return HA_FAIL;
 		}
-
 	}
 	return HA_OK;
 }
@@ -555,7 +681,8 @@ clm_track_cb(SaClmClusterNotificationT *nbuf, SaUint32T nitem,
         for (i = 0; i < nitem; i++) {
 
 	    	cl_log(LOG_INFO, "adding %s in membership table", nbuf[i].clusterNode.nodeName.value);
-                g_array_insert_val(gMembershipTable, i, nbuf[i]);
+
+                g_ptr_array_add(gMembershipTable, (gpointer *) &nbuf[i]);
         }
 
 	if (clmInitialized) {
@@ -584,7 +711,7 @@ int
 init_membership(void)
 {
 	SaErrorT ret;
-	static SaClmClusterNotificationT * nbuf;
+	static SaClmClusterNotificationT * nbuf = NULL;
 	SaClmHandleT handle;
 
 	SaClmCallbacksT my_callbacks = {
@@ -595,15 +722,15 @@ init_membership(void)
 	};
 
 	if ((ret = saClmInitialize(&handle, &my_callbacks, NULL)) != SA_OK) {
-	    cl_log(LOG_ERR, "saClmInitialize error, errno [%d]\n",ret);
-	    return HA_FAIL;
+	    cl_log(LOG_DEBUG, "Membership service currently not available.  Will try again later. errno [%d]",ret);
+	    return HA_OK;
 	}
 
+	if (nbuf)
+		ha_free(nbuf);
+
         nbuf = (SaClmClusterNotificationT *) ha_malloc(gNodeTable->len *
                                 sizeof (SaClmClusterNotificationT));
-	if (nbuf == NULL) {
-                return HA_FAIL;
-	}
 
         if (saClmClusterTrackStart(&handle, SA_TRACK_CURRENT, nbuf,
                 gNodeTable->len) != SA_OK) {
@@ -623,6 +750,7 @@ init_membership(void)
 	clm = handle;
 
 	clmInitialized = 1;
+	cl_log(LOG_INFO, "Membership service initialized successfully.");
 
 	return HA_OK;
 }
@@ -633,6 +761,9 @@ get_membership_fd(void)
     	SaErrorT ret;
 	SaSelectionObjectT st;
 
+	if (!clmInitialized)
+		return 0;
+
 	if ((ret = saClmSelectionObjectGet(&clm, &st)) != SA_OK) {
 	    	cl_log(LOG_ERR, "saClmSelectionObjectGet error, errno [%d]\n", ret);
 		return -1;
@@ -648,9 +779,15 @@ handle_membership_msg(void)
 
 	if ((ret = saClmDispatch(&clm, SA_DISPATCH_ALL)) != SA_OK) {
 		if (ret == SA_ERR_LIBRARY) {
-		    	cl_log(LOG_ERR, "I am evicted.");
+
+		    	cl_log(LOG_DEBUG, "I am evicted.");
+
+			/* mark the membership as uninitialized and try 
+			   again later. */
 			clmInitialized = 0;
-			return HA_FAIL;
+			free_membershiptable();
+
+			return HA_OK;
 		} else {
 		    cl_log(LOG_WARNING, "saClmDispatch error, ret = [%d]", ret);
 		}
@@ -659,15 +796,32 @@ handle_membership_msg(void)
     	return HA_OK;
 }
 
+int
+ping_membership(int * mem_fd)
+{
+	int fd;
+
+	if (clmInitialized) 
+		return 0;
+
+	init_membership();
+	fd = get_membership_fd();
+
+	*mem_fd = fd;
+
+	return HA_OK;
+}
+
 int
 init_resource_table(void)
 {
-    	int rc, i, mcount, found;
+    	int rc, i, count, found;
 	FILE * rcsf;
 	char buf[MAXLINE];
 	char host[MAXLINE], pad[MAXLINE];
-	struct hb_rsinfo resource;
-	char * node;
+	struct hb_rsinfo * resource;
+	struct hb_nodeinfo * node;
+	size_t nodeid = 0;
 
 	if (gResourceTable) {
 	    	free_resourcetable();
@@ -678,6 +832,7 @@ init_resource_table(void)
 		return HA_FAIL;
 	}
 
+	count = 0;
 	for (;;) {
 	    	errno = 0;
 		if (fgets(buf, MAXLINE, rcsf) == NULL) {
@@ -698,44 +853,39 @@ init_resource_table(void)
 		    	cl_log(LOG_WARNING, "%s syntax error?", RESOURCE_CFG);
 		};
 
-		resource.master = g_strdup(host);
-		resource.resource = g_strdup(pad);
-
-		/* make sure that the master node is in the node list */
+		/* make sure that the host node is in the node list */
 		found = 0;
 		for (i = 0; i < gNodeTable->len; i++) {
-		    	node = g_array_index(gNodeTable, 
-				struct hb_nodeinfo, i).name;
-			if (strcmp(node, host) == 0) {
+		    	node = (struct hb_nodeinfo *) g_ptr_array_index(gNodeTable, i);
+			if (strcmp(node->name, host) == 0) {
 			    	found = 1;
+				nodeid = node->id;
 			    	break;
 			}
 		}
 		if (!found)
 		    continue;
 
-		mcount = 0;
-		for (i = 0; i < gResourceTable->len; i++) {
-		    	node = g_array_index(gResourceTable, 
-				struct hb_rsinfo, i).master;
-
-			if (strcmp(node, host) == 0) {
-			    mcount++;
-			}
+		resource = (struct hb_rsinfo *) ha_malloc(sizeof(struct hb_rsinfo));
+		if (!resource) {
+			cl_log(LOG_CRIT, "malloc resource info failed.");
+			return HA_FAIL;
 		}
 
-		resource.index = mcount + 1;
+		resource->id = ++count;
+		resource->masternodeid = nodeid;
+		resource->resource = g_strdup(pad);
 
-		g_array_append_val(gResourceTable, resource);
+		g_ptr_array_add(gResourceTable, (gpointer *) resource);
 	}
 
 	return HA_OK;
 }
 
 int 
-rsinfo_get_int_value(lha_attribute_t attr, size_t index, int32_t * value)
+rsinfo_get_int_value(lha_attribute_t attr, size_t index, uint32_t * value)
 {
-    int rc = 0;
+    uint32_t rc = 0;
     char getcmd[MAXLINE];
     char * resource;
 
@@ -746,10 +896,10 @@ rsinfo_get_int_value(lha_attribute_t attr, size_t index, int32_t * value)
 
     switch (attr) {
 	case RESOURCE_STATUS:
-	    resource = g_array_index(gResourceTable, struct hb_rsinfo, index).resource;
+	    resource = ((struct hb_rsinfo *) g_ptr_array_index(gResourceTable, index))->resource;
 	    sprintf(getcmd, HALIB "/ResourceManager status %s", resource);
 	    rc = system(getcmd);
-	    cl_log(LOG_INFO, "resource [%s] status: [%d]", resource, WEXITSTATUS(rc));
+	    /* cl_log(LOG_INFO, "resource [%s] status: [%d]", resource, WEXITSTATUS(rc)); */
 
 	    *value = WEXITSTATUS(rc);
 	    break;
@@ -765,6 +915,8 @@ rsinfo_get_int_value(lha_attribute_t attr, size_t index, int32_t * value)
 int 
 nodestatus_trap(const char * node, const char * status)
 {
+    uint32_t svalue = 0;
+
     oid objid_snmptrap[] = { 1, 3, 6, 1, 6, 3, 1, 1, 4, 1, 0 };
     size_t objid_snmptrap_len = OID_LENGTH(objid_snmptrap);
 
@@ -779,6 +931,8 @@ nodestatus_trap(const char * node, const char * status)
 
     netsnmp_variable_list *notification_vars = NULL;
 
+    svalue = get_status_value(status, NODE_STATUS, NODE_STATUS_VALUE);
+
     snmp_varlist_add_variable(&notification_vars,
                               /*
                                * the snmpTrapOID.0 variable
@@ -808,12 +962,12 @@ nodestatus_trap(const char * node, const char * status)
     snmp_varlist_add_variable(&notification_vars,
                               nodestatus_oid, 
 			      nodestatus_oid_len,
-                              ASN_OCTET_STR,
-                              (const u_char *) status,
-                              strlen(status)); /* do NOT use strlen() +1 */
+                              ASN_INTEGER,
+                              (const u_char *) & svalue,
+                              sizeof(svalue)); 
 
-    cl_log(LOG_INFO, "sending node status trap. node: %s: status %s", 
-	    node, status);
+    cl_log(LOG_INFO, "sending node status trap. node: %s: status %s, value:%d", 
+	    node, status, svalue);
     send_v2trap(notification_vars);
     snmp_free_varbind(notification_vars);
 
@@ -823,6 +977,8 @@ nodestatus_trap(const char * node, const char * status)
 int 
 ifstatus_trap(const char * node, const char * lnk, const char * status)
 {
+    uint32_t svalue = 0;
+
     oid objid_snmptrap[] = { 1, 3, 6, 1, 6, 3, 1, 1, 4, 1, 0 };
     size_t objid_snmptrap_len = OID_LENGTH(objid_snmptrap);
 
@@ -840,6 +996,8 @@ ifstatus_trap(const char * node, const char * lnk, const char * status)
 
     netsnmp_variable_list *notification_vars = NULL;
 
+    svalue = get_status_value(status, IF_STATUS, IF_STATUS_VALUE);
+
     snmp_varlist_add_variable(&notification_vars,
                               /*
                                * the snmpTrapOID.0 variable
@@ -875,18 +1033,71 @@ ifstatus_trap(const char * node, const char * lnk, const char * status)
     snmp_varlist_add_variable(&notification_vars,
                               ifstatus_oid, 
 			      ifstatus_oid_len,
-                              ASN_OCTET_STR,
-                              (const u_char *) status,
-                              strlen(status)); /* do NOT use strlen() +1 */
+                              ASN_INTEGER,
+                              (const u_char *) & svalue,
+                              sizeof(svalue)); 
 
-    cl_log(LOG_INFO, "sending ifstatus trap. node:%s, lnk: %s, status:%s", 
-	    node, lnk, status);
+    cl_log(LOG_INFO, "sending ifstatus trap. node:%s, lnk: %s, status:%s, value: %d", 
+	    node, lnk, status, svalue);
     send_v2trap(notification_vars);
     snmp_free_varbind(notification_vars);
 
     return HA_OK;
 }
 
+int 
+hbagent_trap(int online, const char * node)
+{
+    oid objid_snmptrap[] = { 1, 3, 6, 1, 6, 3, 1, 1, 4, 1, 0 };
+    size_t objid_snmptrap_len = OID_LENGTH(objid_snmptrap);
+
+    oid  nodename_oid[] = { 1, 3, 6, 1, 4, 1, 4682, 2, 1, 2 };
+    size_t nodename_oid_len = OID_LENGTH(nodename_oid);
+
+  /* this is the oid for the hbagent online trap */
+    oid  trap_oid[] = { 1, 3, 6, 1, 4, 1, 4682, 900, 7 };
+    size_t trap_oid_len = OID_LENGTH(trap_oid);
+
+    /* this is the oid for the offline trap */
+    if (!online) {
+	    trap_oid[trap_oid_len - 1] = 9;
+    }
+
+    netsnmp_variable_list *notification_vars = NULL;
+
+    snmp_varlist_add_variable(&notification_vars,
+                              /*
+                               * the snmpTrapOID.0 variable
+                               */
+                              objid_snmptrap, objid_snmptrap_len,
+                              /*
+                               * value type is an OID
+                               */
+                              ASN_OBJECT_ID,
+                              /*
+                               * value contents is our notification OID
+                               */
+                              (u_char *) trap_oid,
+                              /*
+                               * size in bytes = oid length * sizeof(oid)
+                               */
+                              trap_oid_len * sizeof(oid));
+
+    snmp_varlist_add_variable(&notification_vars,
+                              nodename_oid, 
+			      nodename_oid_len,
+                              ASN_OCTET_STR,
+                              (const u_char *) node,
+                              strlen(node)); /* do NOT use strlen() +1 */
+
+    cl_log(LOG_INFO, "sending hbagent trap. status:%d", online);
+    send_v2trap(notification_vars);
+    snmp_free_varbind(notification_vars);
+
+    return HA_OK;
+}
+
+
 int 
 membership_trap(const char * node, SaClmClusterChangesT status)
 {
@@ -933,7 +1144,7 @@ membership_trap(const char * node, SaClmClusterChangesT status)
                               membershipchange_oid, 
 			      membershipchange_oid_len,
                               ASN_INTEGER,
-                              (u_char *) &status,
+                              (u_char *) & status,
                               sizeof(status)); 
 
     cl_log(LOG_INFO, "sending membership trap. node:%s, status:%d", 
@@ -957,13 +1168,13 @@ main(int argc, char ** argv)
 
 	fd_set fdset;
 	struct timeval tv, *tvp;
-	int flag, block, numfds, hb_fd = 0, mem_fd = 0;
+	int flag, block = 0, numfds, hb_fd = 0, mem_fd = 0, debug = 0;
 
 	/* change this if you want to be a SNMP master agent */
 	int agentx_subagent=1; 
 
 	/* change this if you want to run in the background */
-	int background = 1; 
+	int background = 0; 
 
 	/* change this if you want to use syslog */
 	int syslog = 1; 
@@ -971,7 +1182,7 @@ main(int argc, char ** argv)
 	while ((flag = getopt(argc, argv, "d")) != EOF) {
 	    	switch (flag) {
 		    case 'd':
-			background = 0;
+			debug++ ;
 			break;
 		    default: 
 			usage();
@@ -979,10 +1190,10 @@ main(int argc, char ** argv)
 		}
 	}
 
-	if (background) 
-		cl_log_enable_stderr(FALSE);
+	if (debug) 
+		cl_log_enable_stderr(TRUE);
 	else 
-	    	cl_log_enable_stderr(TRUE);
+		cl_log_enable_stderr(FALSE);
 
 	/* print log errors to syslog or stderr */
 	if (syslog)
@@ -1002,7 +1213,11 @@ main(int argc, char ** argv)
 		exit(1);
 
 	/* initialize the agent library */
-	init_agent("LHA-agent");
+	if (init_agent("LHA-agent")) {
+		cl_log(LOG_ERR, "AgentX initialization failure.  You can only run this as root.  Also make sure that the master snmpd is started and is accepting AgentX connetions.  The subagent will not be respawned.");
+
+		return 100;
+	}
 
 	/* initialize mib code here */
 
@@ -1019,10 +1234,19 @@ main(int argc, char ** argv)
 	    	cl_log(LOG_ERR, "resource table initialization failure.");
 	}
 
+	ret = init_membership();
+	mem_fd = get_membership_fd();
+
+	if (ret != HA_OK) {
+		cl_log(LOG_ERR, "fatal error during membership initialization. ");
+	}  
+
+	/*
 	if ((ret = init_membership() != HA_OK) ||
 		(mem_fd = get_membership_fd()) <= 0) {
-	    	cl_log(LOG_ERR, "membership initialization failure.  You will not be able to view any membership information in this cluster.");
+	    	cl_log(LOG_DEBUG, "membership initialization failure.  You will not be able to view any membership information in this cluster.");
 	} 
+	*/
 
 	init_LHAClusterInfo();
 	init_LHANodeTable();
@@ -1045,6 +1269,8 @@ main(int argc, char ** argv)
 
 	snmp_log(LOG_INFO,"LHA-agent is up and running.\n");
 
+	hbagent_trap(1, myid);
+
 	/* you're main loop here... */
 	while(keep_running) {
 	/* if you use select(), see snmp_select_info() in snmp_api(3) */
@@ -1063,40 +1289,44 @@ main(int argc, char ** argv)
 				numfds = mem_fd + 1;
 		}
 
+		tv.tv_sec = DEFAULT_TIME_OUT;
+		tv.tv_usec = 0;
 		tvp = &tv;
 
-		snmp_select_info(&numfds, &fdset, &tv, &block);
+		snmp_select_info(&numfds, &fdset, tvp, &block);
 
 		if (block) {
 			tvp = NULL;
-		} if (tvp->tv_sec == 0) {
+		} else if (tvp->tv_sec == 0) {
 		    tvp->tv_sec = DEFAULT_TIME_OUT;
+		    tvp->tv_usec = 0;
 		}
 
 		ret = select(numfds, &fdset, 0, 0, tvp);
 
 		if (ret < 0) {
 			/* error */
-			cl_log(LOG_ERR, "select() returned with an error.");
+			cl_log(LOG_ERR, "select() returned with an error. shutting down...");
 			break;
 		} else if (ret == 0) {
 			/* timeout */
+			ping_membership(&mem_fd);
 			snmp_timeout();
-			continue;
+			goto process_pending;
 		} 
 
 		if (FD_ISSET(hb_fd, &fdset)) {
 			/* heartbeat */
 
 			if ((ret = handle_heartbeat_msg()) == HA_FAIL) {
-				cl_log(LOG_ERR, "heartbeat stopped. subagent quit.");
+				cl_log(LOG_DEBUG, "no heartbeat. quit now.");
 				break;
 			}
 		} else  if (clmInitialized && FD_ISSET(mem_fd, &fdset)) {
 		    	/* membership events */
 
 		    	if ((ret = handle_membership_msg()) == HA_FAIL) {
-			    	cl_log(LOG_ERR, "memebership error.");
+			    	cl_log(LOG_DEBUG, "unrecoverable membership error. quit now.");
 				break;
 			}
 		} else {
@@ -1104,10 +1334,18 @@ main(int argc, char ** argv)
 			/* snmp request */
 			snmp_read(&fdset);
 		}
+
+process_pending:
+		run_alarms();
+		netsnmp_check_outstanding_agent_requests();
+
 	}
 
 	/* at shutdown time */
+	
+	hbagent_trap(0, myid);
 	snmp_shutdown("LHA-agent");
+	ha_free(myid);
 	free_storage();
 
 	return 0;
