@@ -1,4 +1,4 @@
-/* $Id: apphbd.c,v 1.51.2.8 2004/09/17 08:58:07 alan Exp $ */
+/* $Id: apphbd.c,v 1.51.2.10 2005/08/19 15:36:22 sunjd Exp $ */
 /*
  * apphbd:	application heartbeat daemon
  *
@@ -90,10 +90,11 @@
 
 #include <clplumbing/cl_signal.h>
 #include <clplumbing/lsb_exitcodes.h>
+#include <clplumbing/coredumps.h>
 
 
 #ifndef PIDFILE
-#	define		PIDFILE "/var/run/apphbd.pid"
+#	define		PIDFILE    HA_VARRUNDIR"/apphbd.pid"
 #endif
 
 const char *	cmdname = "apphbd";
@@ -868,13 +869,14 @@ usage(const char* cmd, int exit_status)
 int
 main(int argc, char ** argv)
 {
-	int	flag;
-	int	req_restart = FALSE;
-	int	req_status = FALSE;
-	int	req_stop = FALSE;
-	int	argerr = 0;
-	const char* cfgfile = CONFIG_FILE;
+	int		flag;
+	int		req_restart = FALSE;
+	int		req_status = FALSE;
+	int		req_stop = FALSE;
+	int		argerr = 0;
+	const char*	cfgfile = CONFIG_FILE;
 
+	cl_cdtocoredir();
 	cl_log_set_entity(cmdname);
 	cl_log_enable_stderr(TRUE);
 	cl_log_set_facility(LOG_USER);
