@@ -1,4 +1,4 @@
-/* $Id: apphbtest.c,v 1.21.2.1 2004/04/20 05:05:37 alan Exp $ */
+/* $Id: apphbtest.c,v 1.21.2.4 2004/09/16 16:42:07 alan Exp $ */
 /*
  * apphbtest:	application heartbeat test program
  *
@@ -36,9 +36,12 @@
 #include <sys/time.h>
 
 #include <clplumbing/cl_log.h>
+#include <clplumbing/cl_log.h>
+#include <clplumbing/timers.h>
+#include <clplumbing/realtime.h>
 #include <apphb.h>
 
-int debug = 0;
+int	debug = 0;
 
 void doafailtest(void);
 
@@ -52,24 +55,29 @@ void apphb_setwarn_test(int warnhb_ms, int hb_ms);
 void dup_reg_test(void);
 
 #define APPNAME_LEN 256
-#define OPTARGS "n:p:i:l:dFh"
+#define OPTARGS "n:p:i:l:dFhR"
 #define USAGE_STR "Usage: [-n heartbeat number] \
 [-p process number] \
 [-l delay seconds] \
 [-i heartbeat interval(ms)] \
 [-d](debug information) \
 [-F](enable failure cases) \
+[-R] (enable realtime)	\
 [-h](print help message)"
 
+/* zeros in first 2 args imply minimums or defaults */
+#define	DOREALTIME cl_make_realtime(0, 0, 8, 50)
+
 int
 main(int argc,char ** argv)
 {
-	int flag;
-	int hb_num = 10;
-	int child_proc_num = 1;
-	int hb_intvl_ms = 1000;
-	int dofailuretests = FALSE;
-	int delaysecs = -1;
+	int		flag;
+	int		hb_num = 10;
+	int		child_proc_num = 1;
+	int		hb_intvl_ms = 1000;
+	int		delaysecs = -1;
+	gboolean	dofailuretests = FALSE;
+	gboolean	dorealtime = FALSE;
 	
 	while (( flag = getopt(argc, argv, OPTARGS)) != EOF) {
 		switch(flag) {
@@ -91,6 +99,9 @@ main(int argc,char ** argv)
 			case 'F':	/* Enable failure cases */
 				dofailuretests = TRUE;
 				break;
+			case 'R':	/* Enable realtime */
+				dorealtime = TRUE;
+				break;
 			case 'h':
 			default:
 				fprintf(stderr
@@ -102,9 +113,15 @@ main(int argc,char ** argv)
 	cl_log_set_entity(argv[0]);
 	cl_log_enable_stderr(TRUE);
 	cl_log_set_facility(LOG_USER);
+	if (dorealtime) {
+		cl_enable_realtime();
+	}else{
+		cl_disable_realtime();
+	}
 	
-	if (delaysecs < 0) 
+	if (delaysecs < 0)  {
 		delaysecs = child_proc_num;
+	}
 	multi_hb_test(child_proc_num, hb_intvl_ms, hb_num, delaysecs
 			, dofailuretests);
 	
@@ -141,16 +158,16 @@ doafailtest(void)
 		cl_log(LOG_INFO, "Client %s registered", app_instance);
 	}
 	
-	cl_log(LOG_INFO, "Client %s setting 2 second heartbeat period"
+	cl_log(LOG_INFO, "Client %s setting 4 second heartbeat period"
 			, app_instance);
-	rc = apphb_setinterval(2000);
+	rc = apphb_setinterval(4000);
 	if (rc < 0) {
 		cl_perror("%s setinterval failure", app_instance);
 		exit(2);
 	}
 
-	for (j=0; j < 10; ++j) {
-		sleep(1);
+	for (j=0; j < 20; ++j) {
+		mssleep(500);
 		if (debug) 
 			fprintf(stderr, "+");
 		if (j == 8) {
@@ -164,7 +181,7 @@ doafailtest(void)
 
 	}
 	if (debug) fprintf(stderr, "\n");
-	sleep(3);
+	mssleep(5000);
 	if (debug) 
 		fprintf(stderr, "!");
 	rc = apphb_hb();
@@ -193,7 +210,6 @@ hb_normal(int hb_intvl_ms, int delaysecs, int hb_num)
 {
 	int	j;
 	int 	rc;
-	struct 	timespec time_spec;
 	char	app_name[] = "apphb_normal";
 	char	app_instance[APPNAME_LEN];
 	struct	timeval tmp;
@@ -201,16 +217,13 @@ hb_normal(int hb_intvl_ms, int delaysecs, int hb_num)
 	sprintf(app_instance, "%s_%ld", app_name, (long)getpid());
 	
 	if (delaysecs) {
+		long	msdelay;
 		/* sleep randomly for a while */
 		gettimeofday(&tmp, NULL);
 		srandom((unsigned int)tmp.tv_usec);
-		delaysecs = random() % delaysecs;
-		if (delaysecs) {
-			cl_log(LOG_INFO, "%s sleep randomly for %d secs"
-				, app_instance, delaysecs);
-			time_spec.tv_sec = delaysecs;
-			time_spec.tv_nsec = 0;
-			nanosleep(&time_spec, NULL);
+		msdelay = random() % (delaysecs*1000);
+		if (msdelay) {
+			mssleep(msdelay);
 		}
 	}
 
@@ -233,13 +246,11 @@ hb_normal(int hb_intvl_ms, int delaysecs, int hb_num)
 		exit(2);
 	}
 
-	/* Sleep for half of the heartbeat interval */
-	time_spec.tv_sec = hb_intvl_ms / 2000;
-	time_spec.tv_nsec = (hb_intvl_ms % 2000) * 500000;
 	for (j=0; j < hb_num; ++j) {
-		nanosleep(&time_spec, NULL);
-		if(debug >= 1) 
+		mssleep(hb_intvl_ms / 2);
+		if (debug >= 1)  {
 			fprintf(stderr, "%ld:+\n", (long)getpid());
+		}
 		rc = apphb_hb();
 		if (rc < 0) {
 			cl_perror("%s apphb_hb failure", app_instance);
@@ -258,6 +269,8 @@ hb_normal(int hb_intvl_ms, int delaysecs, int hb_num)
 	}
 }
 
+
+
 void 
 multi_hb_test(int child_proc_num, int hb_intvl_ms, int hb_num, int delaysecs
 ,	int dofailuretests)
@@ -269,6 +282,7 @@ multi_hb_test(int child_proc_num, int hb_intvl_ms, int hb_num, int delaysecs
 	for (j=0; j < child_proc_num; ++j) {
 		switch(fork()){
 		case 0:
+			DOREALTIME;
 			hb_normal(hb_intvl_ms, delaysecs ,hb_num);
 			exit(0);
 			break;
@@ -291,6 +305,7 @@ multi_hb_test(int child_proc_num, int hb_intvl_ms, int hb_num, int delaysecs
 		for (j = 0; j < child_proc_num; ++j) {
 			switch(fork()){
 			case 0:
+				DOREALTIME;
 				doafailtest();
 				exit(0);
 				break;
@@ -316,7 +331,6 @@ apphb_setwarn_test(int warnhb_ms, int hb_ms)
 	 * warning period, apphbd will warn: 'late heartbeat' */
 
 	int	rc;
-	struct	timespec time_spec;
 	char	app_name[] = "apphb_setwarn_test";
 	char	app_instance[APPNAME_LEN];
 
@@ -349,9 +363,7 @@ apphb_setwarn_test(int warnhb_ms, int hb_ms)
 		exit(4);
 	}
 
-	time_spec.tv_sec = hb_ms/1000;
-	time_spec.tv_nsec = (hb_ms % 1000) * 1000000;
-	nanosleep(&time_spec, NULL);
+	mssleep(hb_ms);
 
 	rc = apphb_hb();
 	if (rc < 0) {
@@ -369,7 +381,8 @@ apphb_setwarn_test(int warnhb_ms, int hb_ms)
 	errno = 0;
 }
 
-void dup_reg_test()
+void
+dup_reg_test()
 {
 	/* apphbd should not allow a process register two times */
 	int	rc;
