@@ -1,4 +1,4 @@
-/* $Id: replica.c,v 1.8.2.2 2004/09/11 06:36:34 msoffen Exp $ */
+/* $Id: replica.c,v 1.8.2.6 2005/01/12 02:25:47 horms Exp $ */
 /* 
  * replica.c: 
  *
@@ -133,7 +133,7 @@ SaCkptReplicaCreate(SaCkptReqOpenParamT* openParam)
 
 	replica->saCkptService = saCkptService;
 	
-	strcpy(replica->checkpointName, openParam->ckptName.value);
+	strcpy(replica->checkpointName, (char*)openParam->ckptName.value);
 	strcpy(replica->activeNodeName, saCkptService->nodeName);
 	replica->flagIsActive= TRUE;
 	replica->createFlag = openParam->attr.creationFlags;
@@ -167,7 +167,7 @@ SaCkptReplicaCreate(SaCkptReqOpenParamT* openParam)
 	replica->retentionTimeoutTag = 0;
 
 	/* create default section */
-	sec = (SaCkptSectionT*)SaCkptMalloc(sizeof(SaCkptSectionT));
+	sec = (SaCkptSectionT*)SaCkptMalloc(sizeof(SaCkptSectionT) + sizeof(SaUint8T));
 	SACKPTASSERT(sec != NULL);
 	sec->replica = replica;
 	sec->sectionID.id[0] = 0;
@@ -368,6 +368,7 @@ SaCkptReplicaPack(void** data, SaSizeT* dataLength,
 {
 	SaCkptSectionT* sec = NULL;
 	SaCkptStateT* state = NULL;
+	int sectionLength = 0;
 
 	GList*	list = NULL;
 
@@ -455,8 +456,12 @@ SaCkptReplicaPack(void** data, SaSizeT* dataLength,
 	list = replica->sectionList;
 	while (list != NULL) {
 		sec = (SaCkptSectionT* )list->data;
+		sectionLength = sizeof(SaCkptSectionT) + sec->sectionID.idLen;
 		index = sec->dataIndex;
 
+		memcpy(p,&sectionLength,sizeof(sectionLength));
+		p += sizeof(sizeof(sectionLength));
+		
 		memcpy(p, &sec->sectionID.idLen, 
 			sizeof(sec->sectionID.idLen));
 		p += sizeof(sec->sectionID.idLen);
@@ -517,6 +522,7 @@ SaCkptReplicaUnpack(void* data, int dataLength)
 	SaUint32T i = 0;
 	SaUint32T n = 0;
 	int m = 0;
+	int sectionLength = 0;
 
 	GList* list = NULL;
 	SaCkptSectionT* sec = NULL;
@@ -591,7 +597,8 @@ SaCkptReplicaUnpack(void* data, int dataLength)
 		memcpy(&(state->state), p, sizeof(state->state));
 		p += sizeof(state->state);
 
-		list = g_list_append(list, (gpointer)state);
+		replica->nodeList = g_list_append(
+			replica->nodeList, (gpointer)state);
 	}
 
 	memcpy(&n, p, sizeof(n));
@@ -606,8 +613,11 @@ SaCkptReplicaUnpack(void* data, int dataLength)
 	p += sizeof(replica->nextOperationNumber);
 
 	list = replica->sectionList;
-	for(i=0; i < replica->sectionNumber; i++) {
-		sec = (SaCkptSectionT*)SaCkptMalloc(sizeof(SaCkptSectionT));
+	for(i=0; i<replica->sectionNumber; i++) {
+		memcpy(&sectionLength ,p,sizeof(sectionLength));
+		p += sizeof(sectionLength);
+		
+		sec = (SaCkptSectionT*)SaCkptMalloc(sectionLength);
 		SACKPTASSERT (sec != NULL);
 		
 		replica->sectionList = g_list_append(replica->sectionList, 
@@ -683,7 +693,9 @@ SaCkptReplicaUnpack(void* data, int dataLength)
 	replica->openCheckpointList= NULL;
 
 	SaCkptFree((void*)&q);
-
+	if(saCkptService->flagVerbose){
+		SaCkptDumpReplica(replica);
+	}
 	cl_log(LOG_INFO, "Replica %s was copied from node %s",
 		replica->checkpointName,
 		replica->activeNodeName);
@@ -772,7 +784,7 @@ SaCkptReplicaUpdate(SaCkptReplicaT* replica, SaCkptReqT req,
 			strSectionID = SaCkptSectionId2String(sec->sectionID);
 			cl_log(LOG_INFO, 
 				"Update: section %s deleted from replica %s",
-				strSectionID,
+				sec->sectionID.id,
 				replica->checkpointName);
 			SaCkptFree((void*)&strSectionID);
 		}
@@ -811,6 +823,13 @@ SaCkptReplicaUpdate(SaCkptReplicaT* replica, SaCkptReqT req,
 					secWrtParam->offset + 
 					dataLength -
 					sec->dataLength[index];
+
+				sec->dataLength[(index+1)%2] = 
+					secWrtParam->offset +
+					dataLength;
+			} else {
+				sec->dataLength[(index+1)%2] = 
+					sec->dataLength[sec->dataIndex];
 			}
 			
 			/* commit the update */
@@ -818,6 +837,7 @@ SaCkptReplicaUpdate(SaCkptReplicaT* replica, SaCkptReqT req,
 			sec->dataLength[index] = 0;
 			sec->dataIndex = (index + 1) % 2;
 			sec->dataUpdateState = OP_STATE_COMMITTED;
+			sec->lastUpdateTime = time_longclock();
 		}
 
 		break;
@@ -845,7 +865,9 @@ SaCkptReplicaUpdate(SaCkptReplicaT* replica, SaCkptReqT req,
 			SaCkptFree((void**)&(sec->data[index]));
 			sec->dataLength[index]= 0;
 			sec->dataIndex = (index + 1) % 2;
+			sec->dataLength[sec->dataIndex] = dataLength;
 			sec->dataUpdateState = OP_STATE_COMMITTED;
+			sec->lastUpdateTime = time_longclock();
 		}
 
 		break;
@@ -902,8 +924,12 @@ SaCkptReplicaUpdPrepare(SaCkptReplicaT* replica, SaCkptReqT req,
 		sec = SaCkptSectionFind(replica, 
 			&(secDelParam->sectionID));
 		if (sec == NULL) {
-			cl_log(LOG_INFO, "Can not find section");
+			if(saCkptService->flagVerbose){
+				cl_log(LOG_INFO, 
+				"Can not find section %s \n", secDelParam->sectionID.id);
+			}
 			retVal = SA_ERR_NOT_EXIST;
+			
 			break;
 		}
 
@@ -917,8 +943,11 @@ SaCkptReplicaUpdPrepare(SaCkptReplicaT* replica, SaCkptReqT req,
 		sec = SaCkptSectionFind(replica, 
 			&(secWrtParam->sectionID));
 		if (sec == NULL) {
-			cl_log(LOG_INFO, "Can not find section");
+			if(saCkptService->flagVerbose){
+				cl_log(LOG_INFO, "Can not find section %s \n", secWrtParam->sectionID.id);
+			}
 			retVal = SA_ERR_INVALID_PARAM;
+			
 			break;
 		}
 
@@ -937,7 +966,10 @@ SaCkptReplicaUpdPrepare(SaCkptReplicaT* replica, SaCkptReqT req,
 		sec = SaCkptSectionFind(replica, 
 			&(secOwrtParam->sectionID));
 		if (sec == NULL) {
-			cl_log(LOG_INFO, "Can not find section");
+			if(saCkptService->flagVerbose){
+				cl_log(LOG_INFO,
+				 "Can not find section %s \n", secOwrtParam->sectionID.id);
+			}
 			retVal = SA_ERR_INVALID_PARAM;
 			break;
 		}
@@ -1045,7 +1077,7 @@ SaCkptReplicaUpdCommit(SaCkptReplicaT* replica, SaCkptReqT req,
 			strSectionID = SaCkptSectionId2String(sec->sectionID);
 			cl_log(LOG_INFO, 
 				"Commit: section %s deleted from replica %s",
-				strSectionID,
+				sec->sectionID.id,
 				replica->checkpointName);
 			SaCkptFree((void*)&strSectionID);
 		}
@@ -1086,6 +1118,13 @@ SaCkptReplicaUpdCommit(SaCkptReplicaT* replica, SaCkptReqT req,
 				secWrtParam->offset + 
 				dataLength -
 				sec->dataLength[index];
+
+			sec->dataLength[(index+1)%2] = 
+				secWrtParam->offset +
+				dataLength;
+		} else {
+			sec->dataLength[(index+1)%2] = 
+				sec->dataLength[sec->dataIndex];
 		}
 		
 		/* commit the update */
@@ -1124,6 +1163,7 @@ SaCkptReplicaUpdCommit(SaCkptReplicaT* replica, SaCkptReqT req,
 		SaCkptFree((void**)&(sec->data[index]));
 		sec->dataLength[index]= 0;
 		sec->dataIndex = (index + 1) % 2;
+		sec->dataLength[sec->dataIndex] = dataLength;
 		sec->dataUpdateState = OP_STATE_COMMITTED;
 		sec->lastUpdateTime = time_longclock();
 		
@@ -1194,7 +1234,7 @@ SaCkptReplicaUpdRollback(SaCkptReplicaT* replica, SaCkptReqT req,
 			strSectionID = SaCkptSectionId2String(sec->sectionID);
 			cl_log(LOG_INFO, 
 				"Rollback: section %s deleted from replica %s",
-				strSectionID,
+				sec->sectionID.id,
 				replica->checkpointName);
 			SaCkptFree((void*)&strSectionID);
 		}
@@ -1308,7 +1348,7 @@ SaCkptSectionFind(SaCkptReplicaT* replica, SaCkptFixLenSectionIdT* sectionID)
 
 	while (list != NULL) {
 		sec = (SaCkptSectionT*)list->data;
-		if (!memcmp(sec->sectionID.id, sectionID->id,
+		if ((sectionID->idLen == sec->sectionID.idLen) && !memcmp(sec->sectionID.id, sectionID->id,
 			sectionID->idLen)) {
 			return sec;
 		}
@@ -1402,14 +1442,17 @@ SaCkptSectionCreate(SaCkptReplicaT* replica,
 	}
 
 	/* create section */
-	sec = (SaCkptSectionT*)SaCkptMalloc(sizeof(SaCkptSectionT));
+	sec = (SaCkptSectionT*)SaCkptMalloc(sizeof(SaCkptSectionT) + secCrtParam->sectionID.idLen);
 	if (sec == NULL) {
 		cl_log(LOG_ERR,
 			"Section create failed, no memory");
 		return SA_ERR_NO_MEMORY;
 	}
 	sec->replica = replica;
-	sec->sectionID= secCrtParam->sectionID;
+	
+	sec->sectionID.idLen = secCrtParam->sectionID.idLen;
+	memcpy(sec->sectionID.id,secCrtParam->sectionID.id,secCrtParam->sectionID.idLen);
+	
 	sec->expirationTime = secCrtParam->expireTime;
 	sec->lastUpdateTime = time_longclock();
 	sec->dataIndex = 0;
@@ -1443,7 +1486,7 @@ SaCkptSectionCreate(SaCkptReplicaT* replica,
 		strSectionID = SaCkptSectionId2String(sec->sectionID);
 		cl_log(LOG_INFO, 
 			"Section %s created in replica %s",
-			strSectionID,
+			sec->sectionID.id,
 			replica->checkpointName);
 		SaCkptFree((void*)&strSectionID);
 	}
@@ -1670,22 +1713,36 @@ SaCkptReplicaNodeFailure(gpointer key,
 char* 
 SaCkptSectionId2String(SaCkptFixLenSectionIdT sectionId)
 {
-	char* strSectionId = NULL;
-	char	buf[3] = {0};	
-	SaSizeT	i = 0;
-
-	strSectionId = (char*)SaCkptMalloc(2*SA_MAX_ID_LENGTH + 3);
+	char* strSectionId = NULL, *pSec = NULL;
+	int i = 0;
+	strSectionId = (char*)SaCkptMalloc(sectionId.idLen * 4 + strlen("\0"));
+	pSec = strSectionId;
 	SACKPTASSERT(strSectionId != NULL);
 
-	strSectionId[0] = '0';
-	strSectionId[1] = 'x';
-	strSectionId[2] = 0;
-	for (i=0; i < sectionId.idLen; i++) {
-		sprintf(buf, "%0x", sectionId.id[i]);
-		strncat(strSectionId, buf, 2);
+	for(i =0; i< sectionId.idLen; i++)
+	{
+		sprintf(pSec, "%d",sectionId.id[i]);
+		pSec ++;
 	}
+	strncat(pSec, "\0", strlen("\0"));
 
 	return strSectionId;
 }
 
-
+void 
+SaCkptDumpReplica(SaCkptReplicaT* replica){
+	GList * list = NULL;
+	SaCkptSectionT * section = NULL;
+	int i = 0;
+	cl_log(LOG_INFO, "\treplica info");
+	cl_log(LOG_INFO, "\tname is %s \n",replica->checkpointName);
+	cl_log(LOG_INFO, "\tsection number is %d \n",(int)replica->sectionNumber);
+	list = replica->sectionList;
+	
+	while(list != NULL){
+		section = (SaCkptSectionT *)list->data;
+		cl_log(LOG_INFO, "\t %d section length is %d, name is %s \n",i++,section->sectionID.idLen,section->sectionID.id);
+		list=list->next;
+		
+	}
+}
