@@ -1,4 +1,4 @@
-/* $Id: response.h,v 1.1.2.1 2004/04/20 05:05:38 alan Exp $ */
+/* $Id: response.h,v 1.1.2.4 2004/11/18 06:54:23 yixiong Exp $ */
 #ifndef _CKPT_RESPONSE_H
 #define _CKPT_RESPONSE_H
 
@@ -14,6 +14,7 @@ typedef struct _SaCkptResponseT {
 
 SaCkptResponseT* SaCkptResponseCreate(SaCkptRequestT*);
 int SaCkptResponseSend(SaCkptResponseT**);
-
+SaCkptResponseT* 
+createLocalReplical(SaCkptRequestT* ckptReq);
 
 #endif
